
#include "phylip.h"
#include "seq.h"

/* version 3.6. (c) Copyright 1993-2000 by the University of Washington.
   Written by Joseph Felsenstein, Lucas Mix, Akiko Fuseki, Sean Lamont,
   Andrew Keeffe, Dan Fineman, and Patrick Colacurcio.
   Permission is granted to copy and use this program provided no fee is
   charged for it and provided that this copyright notice is not removed. */


typedef long vall[maxcategs];
typedef double contribarr[maxcategs];

#ifndef OLDC
/* function prototypes */
void   init_protmats(void);
void   getoptions(void);
void   makeprotfreqs(void);
void   allocrest(void);
void   doinit(void);
void   inputoptions(void);
void   input_protdata(long);
void   makeweights(void);
void   prot_makevalues(long, pointarray, long, long, sequence, steptr);
void   prot_inittable(void);

void   alloc_pmatrix(long);
void   getinput(void);
void   inittravtree(node *);
void   prot_nuview(node *);
void   prot_slopecurv(node *, double, double *, double *, double *);
void   makenewv(node *);
void   update(node *);
void   smooth(node *);
void   make_pmatrix(double **, double **, double **, long, double,
            double, double *, double **);
double prot_evaluate(node *, boolean);

void   treevaluate(void);
void   promlcopy(tree *, tree *, long, long);
void   proml_re_move(node **, node **);
void   insert_(node *, node *, boolean);
void   addtraverse(node *, node *, boolean);
void   rearrange(node *, node *);
void   proml_coordinates(node *, double, long *, double *);
void   proml_printree(void);
void   sigma(node *, double *, double *, double *);
void   describe(node *);

void   prot_reconstr(node *, long);
void   rectrav(node *, long, long);
void   summarize(void);
void   initpromlnode(node **, node **, node *, long, long, long *, long *,
            initops, pointarray, pointarray, Char *, Char *, FILE *);
void   dnaml_treeout(node *);
void   buildnewtip(long, tree *);
void   buildsimpletree(tree *);
void   free_all_protx (long, pointarray);
void   maketree(void);
void   clean_up(void);

/* function prototypes */
#endif

extern sequence y;

long rcategs;


Char infilename[100], outfilename[100], intreename[100], outtreename[100],
     catfilename[100], weightfilename[100];
double *rate, *rrate, *probcat;
long nonodes2, sites, weightsum, categs,
  datasets, ith, njumble, jumb;
long inseed, inseed0, parens;
boolean global, jumble, weights, trout, usertree, reconsider,
  ctgry, rctgry, auto_, hypstate, progress, mulsets, justwts, firstset,
  improve, smoothit, polishing, lngths, gama, invar, userrnaSSUB, userrnaSSUM, userrnaSSUE,userrnaLSUE;
tree curtree, bestree, bestree2, priortree;
node *qwhere, *grbg;
double cv, alpha, lambda, invarfrac, bestyet;
long *enterorder;
steptr aliasweight;
contribarr *contribution, like, nulike, clai;
double **term, **slopeterm, **curveterm;
longer seed;
char *progname;
char aachar[26]="ARNDCQEGHILKMFPSTWYVBZX?*-";

/* Local variables for maketree, propagated globally for c version: */
long k, nextsp, numtrees, maxwhich, mx, mx0, mx1;
double dummy, maxlogl;
boolean succeeded, smoothed;
double **l0gf;
double *l0gl;
double **tbl;
Char ch, ch2;
long col;
vall *mp;


/* Variables introduced to allow for protein probability calculations   */
long   max_num_sibs;                /* maximum number of siblings used in a */
                                /* nuview calculation.  determines size */
                                /* final size of pmatrices              */
double *eigmat;                        /* eig matrix variable                        */
double **probmat;                /* prob matrix variable                        */
double ****dpmatrix;                /* derivative of pmatrix                */
double ****ddpmatrix;           /* derivative of xpmatrix               */
double *****pmatrices;          /* matrix of probabilities of protien   */
                                /* conversion.  The 5 subscripts refer  */
                                /* to sibs, rcategs, categs, final and  */
                                /* initial states, respectively.        */
double freqaa[20];                /* amino acid frequencies                */

/* this rrnaSSUB matrix decomposition due to Elisabeth  Tillier - Andrews Bact10 -July 4 2003 */
static double rrnaSSUBeigmat[20] =
{+0.00000000000000,-6.93511216740675,-6.66387559780579,-5.93752802405227,-9.68182873063751,-0.02950416822431,-3.46764558874275,-3.37010446302091,-3.15560128010754,-0.63735284914842,-0.77359649871739,-0.91063351496400,-1.01850707078182,-1.34342195982348,-1.26684994243455,-1.44016465878825,-2.33762204857521,-2.17896259583103,-1.95749424500274,-2.00146744989583};

static double rrnaSSUBprobmat[20][20] =
{{+0.24101513263024,+0.07223980949773,+0.13133025860037,+0.11914011957278,+0.00158985797669,+0.00271242579721,+0.00588902446999,+0.04097090576526,+0.00178988291523,+0.00089465320754,+0.11401265802488,+0.00094785668839,+0.00597189170417,+0.13451414302325,+0.00253776825511,+0.03157877224850,+0.05851109775841,+0.00080859348458,+0.02875944107365,+0.00478570730604},
{+0.00013734754974,+0.00021013740585,+0.00009013689056,+0.00020261756466,+0.00868495193387,+0.00203344820530,+0.00072021215972,+0.00550554495881,+0.00109304354578,+0.00495241010103,+0.00444310138617,-0.02580517985150,-0.00085295675109,+0.00206213782658,-0.00287920507784,+0.00299801655407,+0.00703906658492,-0.01313856247170,-0.00005900556157,+0.00256273704656},
{+0.00005630296897,+0.00006805440563,+0.00043558030569,+0.00007048164119,-0.03517854653735,+0.00041366489981,+0.00310213530801,+0.00394680354429,+0.00214478893654,+0.00191253391350,-0.00164144678189,-0.00599686359840,+0.00851852606458,+0.00104032445316,+0.01985558065008,-0.00048270426220,+0.00283065258912,-0.00238380641608,+0.00025339717200,+0.00103454074351},
{+0.00002857683052,+0.00016071304690,+0.00013624963060,-0.00002923348191,-0.00062780181825,+0.00135070342124,+0.00049152648018,+0.00082924122445,+0.00243678565160,-0.02922953900558,+0.00623771102374,-0.00415608395827,+0.00043319077218,+0.00833820647874,+0.00106785111363,+0.00063933871265,-0.00084266437166,-0.00290288224990,+0.00965243895943,+0.00598567153974},
{-0.00001895932095,-0.00002081935447,+0.00004253178522,-0.00006004640317,-0.00048038395770,-0.00030856675137,-0.00031897487743,-0.00208983335912,-0.00020099902954,+0.00075586093245,-0.00314835708822,+0.01485351417644,+0.00026913299454,+0.00604025274455,+0.00105998719342,+0.00012203511088,+0.00703987737228,-0.02486347660744,+0.00148580509595,-0.00015858065647},
{-0.21986365558220,-0.06163005184890,-0.11149955608031,-0.10279986262888,+0.00172103068448,+0.00304075579140,+0.00646984366295,+0.04670099930753,+0.00197711712446,+0.00099711030050,+0.12876644486770,+0.00102106866894,+0.00660310549656,+0.15411605497788,+0.00281229343641,+0.03622842901909,+0.06667183983501,+0.00089687885811,+0.03251887342603,+0.00525128068296},
{-0.00004920250668,-0.00006350701342,+0.00015747886825,+0.00003004781936,-0.00922831782842,+0.00312226744861,+0.00311127836876,+0.00939975110375,+0.03084912581709,+0.00096794730221,-0.00348837384903,+0.00057782615843,+0.00457424780156,-0.00353883134972,-0.03184528777060,+0.00624887248505,-0.00833381828118,-0.00019653479900,-0.00208386591303,-0.00021110386198},
{+0.00017652762907,+0.00007806860272,+0.00070119260359,-0.00000600059334,-0.01184305046021,+0.00592358021083,+0.00495611803279,-0.00859845342607,-0.02821950457355,-0.00085300158502,+0.01526385478878,-0.00111890125053,+0.00965207460876,+0.00517069162721,-0.03236559167841,+0.01044958822677,+0.01721276488242,-0.00073638186007,+0.01177251993997,+0.00238390427424},
{-0.00004993223905,+0.00017482126199,+0.00000093950419,-0.00018627813978,-0.00009328573136,-0.00200222093715,-0.00057428206198,+0.18575418831380,-0.00362752410876,-0.00038677347586,-0.01176573626825,+0.00047871982119,+0.00006872427067,-0.09268183059773,-0.00102257919734,-0.01244414477028,-0.06408699018410,-0.00042414513047,+0.00799662043002,-0.00512829075976},
{-0.36011565584187,+0.08937181010783,+0.09955634009870,+0.18120398093716,-0.00004885198846,-0.00009624097910,-0.00047176856756,-0.00102562173494,-0.00003415288629,-0.00000774655987,-0.00135594651191,-0.00000530709716,-0.00039130263667,-0.00379834549958,-0.00007291283425,-0.00109514191061,-0.00129685285726,-0.00001309047844,-0.00038068263864,+0.00007748987891},
{-0.06579963951168,-0.09628724476616,+0.30229796539374,-0.13700924331546,+0.00013068846540,+0.00000846741922,+0.00068680760006,-0.00066161064394,+0.00004692018988,-0.00000112300563,+0.00017019475524,-0.00000138021538,+0.00070488609469,-0.00359497122039,+0.00009881116191,-0.00058288090865,-0.00028732049098,+0.00000045402190,+0.00021703042147,-0.00013681144523},
{-0.00197419024103,+0.00135006603561,-0.00439980238823,+0.00276300431707,+0.00584764212723,+0.00473081825480,+0.05638269622583,-0.01022864129062,+0.00117290034159,+0.00003874828451,-0.03404299931087,+0.00084058232953,+0.04650083445900,-0.06999809492078,+0.00497503840374,+0.01126397581193,-0.01292465920824,+0.00027538807093,-0.00364952798116,+0.00107622067920},
{+0.00143692178409,-0.00118871556240,-0.00333634048155,+0.00085160238008,+0.00007089293936,-0.00301467824711,-0.00323997374976,-0.01905996989400,+0.00206044972227,+0.00042231217923,+0.18978861084928,+0.00026601073658,+0.00120666893192,-0.22847541035881,+0.00011875875908,-0.06015777734960,+0.07373882498592,+0.00003663275152,+0.04395788409084,+0.00451729553306},
{-0.02381784223162,+0.21427183924949,+0.01795445155389,-0.20667114938533,+0.00000324037010,-0.00013008375539,+0.00030421834982,-0.00028941394382,-0.00000233792278,-0.00000543577750,+0.00195771493247,-0.00001418759896,+0.00034581906341,+0.00325279966132,-0.00008949123030,-0.00526897394367,-0.00037945594816,-0.00000615807079,-0.00089619149562,-0.00051936187655},
{-0.00018983605392,+0.00052624981941,+0.00045683067145,-0.00077539251819,-0.00299445626232,+0.00275794126513,+0.04825255599277,-0.00029576375890,-0.00067164226943,+0.00004036825737,+0.01401222090571,+0.00012835230857,-0.05922585660485,-0.01723974283250,-0.00074576264005,+0.01015501824685,+0.00181470351030,-0.00011319437699,+0.00502028178621,-0.00091287544661},
{+0.00065287851705,-0.00678774986299,-0.00070255888999,+0.00661533742040,-0.00028450822380,-0.00565932950955,+0.01344966783258,+0.00885932208769,-0.00061662689778,-0.00013016059275,+0.00371475569646,-0.00050115980984,+0.00350322386405,+0.15637185518990,-0.00297614843543,-0.15436733569819,-0.00147490007184,-0.00012800168230,-0.01529303825747,-0.00424552267622},
{-0.00035855289023,+0.00055901830867,+0.00077812601199,-0.00018006946216,+0.00003187181631,-0.00185938174755,+0.00034581923899,+0.04173319060999,+0.00024328377249,-0.00118162666143,-0.11530374121696,+0.00024022043378,-0.00155782303858,-0.04337360794407,-0.00070244689022,-0.00161303643202,+0.19325403487019,+0.00068677476214,-0.07277489307549,+0.00103283953416},
{+0.00010912165888,+0.00035250428438,+0.00019485284552,-0.00193131078553,-0.00004520448474,-0.00634943343052,+0.00071001784415,+0.00797353674826,-0.00045734857394,+0.00179124656420,+0.01737196456820,+0.00118088448311,-0.00200102409534,-0.00293040530981,-0.00063483567785,-0.00460652834167,-0.03472402392366,+0.00035376124711,-0.04172300583073,+0.06536523021004},
{+0.00013753071978,-0.00020724561815,+0.00005841449317,+0.00064122424550,-0.00144435325771,-0.04575635845804,+0.00789208892117,-0.00633077554671,-0.00141369235482,-0.00129180637917,+0.07891438706349,-0.00131635724318,+0.00370754191343,+0.00869101865118,-0.00438599292749,+0.03462492305103,-0.01489957110174,-0.00074632189835,-0.04359401268596,-0.01328064158750},
{+0.00002341575537,-0.00134142665673,-0.00010214299286,+0.00125588917683,-0.00001432800116,+0.02078951855502,-0.00224110272574,+0.00029252279931,-0.00105184876786,-0.00120454890808,+0.16358309301534,+0.00031762252228,+0.00052784418358,-0.00862372654339,+0.00125954774200,+0.00510568481190,-0.04092670065125,-0.00043651413294,-0.12421517890028,-0.01299762028132}};


/* this rrnaSSUM matrix decomposition due to Elisabeth  Tillier */
static double rrnaSSUMeigmat[] =
{-0.00000000000001,-4.46454860326932,-3.69633435495133,-3.39181151853187,-5.62438849046521,-0.07456752867679,-2.90714810962679,-2.74789854393423,-0.23566358728649,-2.32416365535794,-2.06234161271743,-1.91468453513485,-0.75997426173509,-0.83815974240889,-0.95724882430993,-1.10924251944115,-1.59552296913579,-1.54078631702494,-1.43505155143359,-1.22367072676009};
static double rrnaSSUMprobmat[20][20] =
{{+0.27973407995779,+0.13252329453788,+0.06950674791414,+0.11677219442323,+0.01510367270589,+0.01055645631249,+0.00633454299769,+0.05767300355190,+0.01653909512609,+0.00676641780078,+0.08157093480901,+0.00874955530982,+0.00620130324671,+0.07520560814208,+0.00288158566413,+0.01333244060693,+0.06937605116192,+0.00392879924684,+0.01784633033233,+0.00939788615235},
{+0.00030126049634,-0.00062760054627,+0.00004377657996,-0.00082508117667,+0.01415951106162,-0.07128547300312,-0.00211022678189,+0.02146257826788,-0.00510590911387,+0.04737142375907,+0.00073906049885,-0.02883526773990,-0.00118393980353,+0.01031173484174,+0.00007591994078,+0.00283716227713,-0.00470496907885,+0.01605107569588,-0.00020353519726,+0.00152849902211},
{-0.00180168484402,+0.00334448607209,+0.00056817655265,-0.00055564674578,+0.07030827995890,-0.03874529597603,-0.03273445328931,+0.01169642593033,-0.01518112201103,-0.02372223872112,+0.00448045833277,+0.03621173424972,-0.01370175769967,+0.00751179684129,+0.00591147150516,+0.00181871129794,+0.00041546174663,-0.01604879183653,+0.00421742679487,-0.00399343815885},
{-0.00211863225880,-0.00337543507728,+0.00087316338201,-0.00044471111373,+0.04540483774118,+0.05256898706053,-0.03974350338042,-0.02518231506321,-0.00736304857427,+0.01687039013300,+0.00716641517400,-0.03810204606300,-0.01580925659612,-0.01130807466497,+0.00897114779981,-0.00201088879199,-0.00962661130723,+0.01466518425200,+0.00412979128488,+0.00443460606367},
{-0.00023426528791,+0.00084246269915,-0.00001871842188,-0.00237662437971,-0.00036763576708,-0.01092494280359,-0.00013816664700,+0.00294427803001,-0.00133653229900,-0.03685989156879,+0.00081207094696,+0.01190399335994,+0.00002791022901,+0.00098280600273,-0.00029764916373,+0.00030785905396,-0.00916128997247,+0.05461224846808,-0.00057067185179,-0.01014724062697},
{-0.23259371530500,-0.10531632641016,-0.05617271436690,-0.09348270634418,+0.01453692025035,+0.01202192688355,+0.00638768908053,+0.07487268792163,+0.01633533002093,+0.00640500300676,+0.09857085873596,+0.00864692706290,+0.00664571794776,+0.11111470624510,+0.00316694166623,+0.01785370399790,+0.07783538588361,+0.00364251344269,+0.02013539937998,+0.00939375090039},
{-0.00093331461606,+0.00333757713429,-0.00036486189390,-0.00164730958437,-0.00485713285130,-0.01904134062380,+0.00045062555412,+0.01543112993381,+0.08831590549242,-0.02914776773560,-0.00761203284854,-0.03613387861211,-0.01731598793052,+0.00444795172073,+0.01309215920826,+0.00046276073310,-0.02550568408611,-0.00684483502096,-0.00396738236243,+0.02783341838898},
{+0.00165999759084,-0.00231766696526,-0.00133459679589,+0.00167262956828,-0.01817532110227,+0.00345177673045,+0.02225335352035,-0.00250985876562,-0.01773351028549,+0.01298397982601,-0.00757681123025,+0.01911070213209,-0.04114075083133,-0.00115493558278,+0.03786919835795,+0.00107999974499,+0.01616898295762,+0.00249649126636,-0.00978885601658,-0.01701480411941},
{+0.01798260513960,+0.00756995475677,+0.00285526734401,+0.00639545770042,-0.00412356900475,+0.00892501126414,-0.00445332224735,+0.07180237663326,-0.01418918630012,-0.00051891880012,-0.15740619029947,-0.00176300192688,-0.00070852374696,+0.16964325276297,-0.00100097913611,+0.02035147474946,-0.09108939533154,-0.00079215895627,-0.02879624722379,-0.00068390737729},
{-0.00032022126043,-0.01525765446233,+0.00105435761259,+0.01272071694285,-0.02536256539211,+0.00331666973521,-0.03024306015811,+0.00466079753813,+0.07081889242197,+0.02084120639934,-0.00732025439535,+0.02638014811468,+0.00849174325421,-0.00276917878267,+0.00117193394286,+0.00188546998841,-0.02359890399273,+0.00160123188663,+0.00811941452412,-0.05619074391724},
{+0.00547084506054,+0.01695265224766,+0.00215516790403,-0.01710147618679,-0.05911423932629,-0.01087716093476,-0.03886774885851,+0.02378911278079,-0.02890766424260,-0.00437814388001,+0.00684271250502,+0.00372909468607,+0.02817930273415,+0.00455352542585,+0.02204130326412,-0.01829860545479,+0.02053761530778,-0.00040605915448,+0.01099247589467,+0.03270729022753},
{+0.00548487186387,+0.24203425596026,+0.00571915976377,-0.25319069725784,+0.00178388056487,+0.00168110266228,+0.00102000831834,-0.00205803085502,+0.00475422691453,+0.00213806201084,-0.00246046961394,+0.00089324487545,-0.00195307917433,-0.00113991283802,-0.00140281623712,+0.00233801105522,-0.00068647983589,-0.00003816384691,+0.00097421945080,-0.00589139378113},
{+0.04089931054731,-0.02106197365733,+0.02571780888166,-0.01739425214773,-0.02232909146544,-0.01094301949437,-0.00484047940272,-0.09578890568680,-0.01958471252249,-0.01107311546476,+0.14781349882634,-0.01690545528454,-0.01074789888221,+0.13394589823799,-0.00294693775435,-0.01354102335023,-0.08174614436772,-0.00594322852387,+0.00751965426243,-0.02104993275120},
{+0.20579695743060,-0.19930133521612,+0.14577354834072,-0.16032281534443,+0.00494006783073,+0.00133651249148,+0.00209057341493,+0.01832782401174,+0.00180879498340,+0.00025278254525,-0.01912062267808,+0.00083090038117,+0.00294126268561,-0.02090943290712,+0.00125707700770,+0.00350505520780,+0.00873208347458,+0.00015618097168,+0.00072488178568,+0.00117970358270},
{-0.00597776187455,+0.00725976387258,-0.00146824120643,+0.00403288880184,-0.00481521956025,+0.00666557995962,+0.00117756819057,+0.13162482375073,-0.01420283524619,+0.00028212959429,+0.11383869957734,+0.00093495117811,-0.00252209551239,-0.09414068737309,+0.00022113090258,+0.02184151335316,-0.15921658062793,-0.00157605015128,-0.00432429417483,+0.00036471654613},
{+0.32073418906627,-0.06590258261282,-0.19841907483511,-0.06012803714229,-0.00072096098516,-0.00006748906216,-0.00117995386904,+0.00420180152523,-0.00024089051118,-0.00048734210883,+0.00534170999875,-0.00057588291409,-0.00112327014940,-0.00002873172681,-0.00082927811026,-0.00082344707367,+0.00372097644262,-0.00023797762741,-0.00249548039555,-0.00073827790910},
{+0.00566336497892,-0.00366028064946,-0.00570214444489,+0.00234084844767,-0.01202300726130,-0.00132423147851,+0.00153266923807,-0.01973431391788,-0.00501720396672,+0.00137232534316,-0.06245003642961,+0.00448003453087,-0.01454232382322,-0.00370811761108,-0.00370023687326,+0.02800538683269,-0.04216010065748,+0.00072466791063,+0.11906721055809,+0.01083548927334},
{+0.00362886414630,-0.00375894339634,+0.00221411551905,-0.00066925353767,-0.02059844785346,-0.00387809419934,-0.01368084396007,-0.07820902316192,+0.00188935409083,+0.00373531102431,+0.02736187481354,+0.01123401380484,-0.00706731815745,-0.01614927917823,-0.00384181630166,+0.09366070819023,+0.02015030532441,+0.00182934750016,-0.03838802032009,+0.02053714565257},
{-0.00339162571948,+0.00901022232282,-0.00238531029516,+0.01169507527864,-0.00363769457609,-0.00768180916351,-0.00047740692637,+0.04361635709982,-0.02746953926741,-0.02457333502237,-0.01592542711642,-0.04509322601074,+0.01349025415988,-0.02346840056474,+0.00775366197328,+0.03927755500928,+0.07831020850480,-0.00939521934291,+0.01297167723957,-0.05262601758285},
{-0.00022371897591,-0.00137109840032,+0.01199341435396,+0.00201621001258,-0.04263913570844,-0.00260101818837,-0.02088294230960,+0.07903231957049,-0.01118763714292,-0.00321419427702,+0.00502318778961,-0.00316764411149,-0.04734448158215,-0.00393732225973,-0.02350566416907,-0.01697887324860,+0.08762925235909,-0.00092685181246,-0.00760254023113,-0.00011126166853}};

 
static double rrnaSSUEeigmat[20] =
{-0.00000000000001,-4.21142062136450,-0.07316419857853,-3.63479500746059,-3.49778126213983,-3.22785579783240,-3.10829813841034,-2.53835888433990,-2.49028977724861,-2.32780469083613,-0.81629678855753,-0.79441722808175,-0.98414885968519,-0.95337394996240,-1.77077919705715,-1.73862068904871,-1.65886217165313,-1.35279869446841,-1.50592893294907,-1.42425168268968};
static double rrnaSSUEprobmat[20][20] =
{{+0.22959563000411,+0.09183584203809,+0.13263878419057,+0.17000530558372,+0.00446511996345,+0.00520180227726,+0.00339432619389,+0.05543243758785,+0.00457803341941,+0.00365064210129,+0.07607730176041,+0.00522019697232,+0.00643772088732,+0.09240293470491,+0.00435566441007,+0.02808312224796,+0.05202820298957,+0.00329675172511,+0.02216876205006,+0.00913141889263},
{-0.00007946240319,+0.00027906894411,+0.00022622490130,+0.00002641508879,+0.00770700280391,+0.02206616443273,-0.00415476452163,+0.00135306297174,-0.00085325631616,+0.00300043815223,+0.00083812483679,+0.00599824076866,-0.00069505175732,+0.00083623427710,+0.00510950460830,-0.00140900228331,+0.00361612847591,-0.05341398390098,+0.00185774507647,+0.00769116584451},
{-0.18715222609734,-0.06764059304953,-0.10037429675097,-0.12894648464144,+0.00566809679941,+0.00655267887775,+0.00416102475502,+0.07342480677223,+0.00582468263777,+0.00440476760505,+0.09742864665206,+0.00637938972024,+0.00829412659384,+0.12041893325965,+0.00516359489796,+0.03547582831822,+0.06859310578931,+0.00405628701023,+0.02720748927806,+0.01106014157256},
{-0.00033343317770,-0.00027762111750,-0.00021235832731,-0.00020913644350,-0.02406849671676,+0.00605719775227,+0.05348240688838,-0.00930820357292,+0.00240584574973,+0.00013237485080,-0.00766306624282,-0.00353514615784,-0.00005875093640,-0.00282997760036,-0.00400414289896,-0.00221624053112,+0.00201850878251,-0.00556112107366,-0.00475574484402,+0.00093710561721},
{+0.00009571737821,-0.00035463500399,-0.00014719786951,+0.00029637615737,+0.01331676614632,-0.06207367634518,+0.00940313839198,+0.01370126463025,-0.00799059515831,+0.00775691289227,-0.00506059198864,+0.02275335003011,-0.00068357606798,+0.00988761994935,+0.00424887486927,+0.00697203899587,+0.00078919965789,-0.01319355151352,+0.00170977403558,-0.00142720918735},
{-0.00006761795569,+0.00003997236540,-0.00019155516855,+0.00022456143779,-0.00629147991905,+0.01802230552029,+0.00024601194831,-0.00875348100107,-0.03537598051472,-0.01273717680993,+0.01138204559732,+0.05259993749196,+0.00106624039931,+0.00091313322617,-0.00685546547398,-0.00348824130902,+0.00952140984692,+0.00589588286647,-0.00408248650819,-0.02206801603970},
{-0.00005417448784,-0.00155932305745,-0.00097836176821,+0.00009637959850,-0.01824801638054,-0.00038861096568,-0.00222418608073,+0.00208810167939,-0.00328973196367,-0.00404126231985,-0.01103745497474,+0.00222597268032,-0.00094586745506,-0.01108508150924,+0.06279291470514,-0.01046360909237,+0.00170810356800,+0.00249979397394,-0.00372138704975,-0.00337419910010},
{-0.00020684586978,+0.00139743243306,+0.00036427546816,-0.00088216858113,+0.00971042586691,-0.00482486283192,+0.00306726412763,-0.00158791809045,+0.03137657432337,-0.05016584492219,-0.00794147765238,+0.01371916812613,-0.00160864944754,+0.01554178582692,+0.00063151683216,-0.00122304174836,-0.01694493409475,-0.00170644582340,-0.00238213192235,+0.01366587797996},
{+0.00009384813454,-0.00082348459724,-0.00040071531682,+0.00126969968540,-0.01844043939360,+0.00205294018553,-0.00677242878715,+0.01438191109107,+0.04460183873857,+0.01984415502232,-0.02218315919585,+0.02806077568906,+0.00088666577490,-0.00489145242615,-0.00384077535745,+0.00192772382710,-0.01380259512868,-0.00098025376162,+0.00154699847683,-0.04253125266079},
{+0.00042369767691,+0.00242550668248,+0.00055848460977,-0.00206040319028,-0.05172477869851,-0.01631651278546,-0.01765444227544,+0.02312569366679,-0.00604825609969,-0.00857622636416,+0.00395305181829,-0.00187219492562,+0.00841050554324,+0.01276892196880,-0.01453470003934,+0.01049992945273,+0.01114904473491,-0.00824287157932,+0.02222628626988,+0.03148926353405},
{+0.32457407301577,-0.10890928804054,+0.02673057339291,-0.25274650069911,+0.00011262758974,-0.00072262902582,+0.00008860275260,-0.00744268903846,+0.00130632783367,-0.00032802293507,+0.02880512563449,+0.00021061000517,-0.00059176510891,-0.02664883077195,-0.00013822938533,-0.00812607066039,+0.01813602153137,-0.00014179575469,+0.00657419267230,-0.00074233300771},
{-0.05552899209271,+0.02024108551488,-0.00967985602131,+0.04481652470570,-0.00055868455415,-0.00458416214549,+0.00000046480058,-0.05699864457291,+0.00677762337345,-0.00121789041222,+0.15288148026165,+0.00164522714649,-0.01067854480680,-0.16077759000741,-0.00072658068681,-0.04801411962633,+0.08668371640164,-0.00068539737191,+0.03774155527207,-0.00133721517841},
{+0.18084450610831,+0.05223797858027,-0.32261137030249,+0.08495146701438,-0.00000106418414,+0.00012986284660,+0.00000288062555,+0.00376728152683,-0.00006834638572,-0.00000464665521,-0.00173777578437,-0.00004259425338,-0.00102744594299,+0.00399850108419,-0.00012094898679,-0.00007113156021,+0.00060754193254,+0.00002046130506,-0.00116047281432,+0.00028531584588},
{-0.00184647094779,+0.00211030211307,-0.00589738408036,+0.00476808509912,+0.00571960627941,+0.00011305923144,+0.00166984973349,-0.02276237207980,+0.00152608418359,-0.00040399213540,-0.01008950131920,+0.00031775190681,+0.07781810589918,-0.05858539537880,+0.00190047353026,-0.00358873278351,+0.00507573864006,+0.00003886270682,+0.00312531138683,-0.00100938198523},
{+0.02631681170149,-0.24864979882892,-0.00395183451445,+0.22214748295294,+0.00017668726534,+0.00104591254479,-0.00009095885925,-0.01261786798318,+0.00197387744542,+0.00301064808169,+0.00255937284571,+0.00460351322331,-0.00003193172424,+0.00624381939211,+0.00005118903800,-0.00680596657933,-0.00262701544365,+0.00133136506709,-0.00736069851788,+0.01267539289301},
{-0.00391614242808,+0.04436119798350,+0.00041032416859,-0.04420037014308,+0.00273816641370,+0.00631496445753,+0.00029414183347,-0.06296215305284,+0.01081471782413,+0.01839531497040,-0.00555849974520,+0.02571766276627,-0.00109305580703,+0.01841326096748,+0.00217317260717,-0.03634228913892,-0.00655366741693,+0.00795472519367,-0.04342335611116,+0.06646188465731},
{+0.00139780722963,-0.00083963367202,-0.00317826656693,-0.00083493681539,-0.00131927645384,+0.00040144164603,-0.00106690794742,-0.10190930609632,+0.00235746551780,+0.00033013650235,+0.00981198378096,+0.00226977378783,-0.00484928143864,-0.05843175781830,+0.00299206523665,+0.14352253899669,+0.02570431418529,+0.00001497038738,-0.01998642324367,+0.00361329278190},
{-0.00150192220432,-0.00371715946260,+0.00253685709353,+0.00387682265542,+0.00570334915183,+0.00582250176800,+0.00236812865363,+0.13726185117552,+0.00136692767485,+0.00170603453198,-0.11377146394785,+0.00747939545015,-0.00597911441360,-0.16162996643302,-0.00049100934542,+0.02191917981354,+0.07471833258316,+0.00389250126182,-0.00179046578512,+0.02022921977850},
{+0.00139579254150,-0.00147526217667,-0.00279478309039,-0.00228814842826,+0.00571972188306,+0.00395788203321,+0.00244130997150,-0.07311970404846,+0.00058187678262,+0.00352804350823,-0.12106626058865,+0.00590785382825,-0.00426300817874,+0.03419197403975,+0.00252017664878,-0.00087721762910,+0.01208090420349,+0.00327559310266,+0.12050172404307,+0.00978153155416},
{-0.00106975581177,-0.00010558280825,+0.00279086357805,+0.00406667662186,-0.00266637317419,-0.00491078454809,-0.00344725416677,-0.05774362282158,+0.00122876981849,-0.00463994944737,-0.08858893565363,-0.00733363674192,+0.00081184030138,+0.08724526948591,-0.00300628380605,-0.03019949110739,+0.17203054274148,-0.00266995532391,-0.04509570431564,-0.01669663282062}};

 
 static double rrnaLSUEeigmat[20] =
{+0.00000000000001,-6.01422982786152,-5.11588518148236,-4.30494276919387,-8.58650584052014,-0.12501503219164,-3.17876879763858,-3.00368212656059,-0.77031152870012,-0.71519111756115,-0.99755040325696,-2.23322913716092,-1.27048138014286,-1.34652254000798,-2.05356110568718,-1.47001267267631,-1.63876670645031,-1.90605325277836,-1.72919477564845,-1.82318827682866};

static double rrnaLSUEprobmat[20][20] =
{{+0.22052257172730,+0.08512003094635,+0.14245668518352,+0.13155926938274,+0.00295821187936,+0.00342443657515,+0.00509748456883,+0.05906098367409,+0.00498328334059,+0.00385861946136,+0.09340129300521,+0.00321120033274,+0.00594937481374,+0.10848731162525,+0.00586041423913,+0.02647297216989,+0.06012996536210,+0.00531053896215,+0.02420196799994,+0.00793338475056},
{+0.00103616138206,+0.00079742973979,+0.00407206021361,+0.00311686992159,+0.00076833383778,+0.01333861823034,+0.00674011472040,-0.00218463983403,+0.00123874282276,-0.00459639105324,+0.00596132852014,+0.00911110469932,+0.00487806913382,+0.00559227907865,-0.07236370598566,+0.00780790522404,-0.00010235600175,+0.00377571553324,+0.00776315240599,+0.00324920741118},
{+0.00008173442815,-0.00089195550705,+0.00001196465584,-0.00051619885201,-0.00558829347785,+0.04944994641843,-0.00215498855750,-0.00807222291810,-0.00045884146866,+0.00388642118222,+0.00139325095845,-0.02867518866168,-0.00190768100439,-0.01128565525148,+0.00799865389468,-0.00239427913628,+0.00040312248737,-0.00142432415570,+0.00005174739351,+0.00009278757209},
{-0.00074723103475,-0.00240689167399,-0.00290959467329,-0.00273677398143,-0.00776416214994,+0.02418342456057,-0.00662674278605,-0.00748567340500,-0.00207946602465,+0.00977364940086,-0.00869074532542,+0.04538499197346,-0.00216623788265,-0.00865278504452,+0.01416516473145,-0.01156837905561,+0.00220805284355,-0.02130131853670,-0.00503974120724,-0.00553954072864},
{-0.00009065759887,+0.00105818079564,+0.00090500871652,+0.00136506731823,-0.00054405351719,+0.00865047018281,-0.00028572015000,+0.00017479550530,+0.00102931354244,-0.05837505811038,+0.00499072663338,+0.00877053751214,-0.00079684669043,+0.00329596335960,+0.01134409613290,+0.00033245125944,-0.00006469233514,+0.01594348916562,+0.00053668710285,+0.00176024117513},
{-0.20837533957210,-0.06462917778653,-0.11576045103233,-0.10299563259722,+0.00273815391360,+0.00336313085869,+0.00463845675513,+0.07321837640783,+0.00531119682316,+0.00329113243887,+0.11147324779785,+0.00272373554153,+0.00596631623277,+0.13144860354759,+0.00466116868110,+0.02941436673894,+0.07439755641384,+0.00501451238633,+0.02632406843883,+0.00777657801204},
{-0.00071058254979,+0.00009703672559,-0.00022065292908,+0.00034948115163,+0.04956552202522,+0.00539982800837,-0.01463926061126,-0.00542681610413,-0.00048473461843,-0.00398164793728,+0.00270257869940,-0.00047765661550,-0.01195123118225,+0.00492642275035,-0.00056654994886,-0.00007406286686,-0.00176911782713,-0.02175081595041,-0.00278707755718,+0.00179933733767},
{-0.00040133396748,-0.00550382795544,-0.00229138679314,-0.00222156040487,+0.01285269573625,+0.00697613777841,-0.01315858551391,-0.00630238908889,-0.00499815612600,+0.01543941996343,-0.00665120321343,+0.01023826091747,-0.00580790082224,-0.01794085175566,+0.00383120151421,-0.00727413623364,-0.00823563011163,+0.06250884907282,-0.01395423462838,-0.01710536836786},
{-0.35290244634247,+0.08107510538657,+0.11536670589761,+0.18377576825007,+0.00005630343815,+0.00028243347723,+0.00024720599422,-0.00165542390395,-0.00043823661332,+0.00045744291111,-0.01536554799989,+0.00031232811596,+0.00047686173373,+0.00281017409746,+0.00070060007300,+0.00219876489088,-0.01662572819539,+0.00033024280443,-0.00166159077858,+0.00055903676317},
{-0.02362196197601,+0.00562232742947,+0.00568217177335,+0.01361031066300,+0.00038124156723,-0.00247932724232,+0.00038424434884,-0.08553426673148,+0.00626093079005,-0.00021598755747,+0.14282258309705,-0.00016132011312,-0.00176953548650,-0.17758714103791,-0.00063111375888,-0.03717887879076,+0.12663226301402,+0.00038521820603,+0.02830829762957,-0.00091005582418},
{+0.05052238525718,+0.07017093204666,-0.30713557218249,+0.18782212835735,-0.00002735074982,+0.00014217089285,+0.00014619593601,+0.00241981747651,+0.00027224956746,+0.00008669669520,-0.00187293408911,+0.00000859220394,-0.00044776400829,-0.00095722567390,-0.00027107104184,+0.00005191854214,-0.00056444596815,+0.00020336143747,-0.00064152804057,+0.00007144334140},
{+0.00178306754624,+0.00118806151937,+0.00087737512102,+0.00188715399820,-0.00949369669684,-0.00344385503469,-0.06320051554466,+0.00715604095847,+0.00618481927292,-0.00091813208508,-0.00124972486285,-0.00210790411406,+0.01077297880564,-0.00375382124169,-0.00571894536832,+0.00862220388226,-0.00695490800609,-0.00020758336242,+0.02931946760746,+0.02925791760514},
{-0.00336268278493,-0.00275605377154,+0.00675126866401,+0.00824696242472,-0.00031475966732,-0.00014114007045,-0.00400054609044,+0.05848613004001,+0.00394450603830,-0.00133877494171,-0.18400623171362,-0.00140347716386,-0.00229036492368,-0.01650564242479,-0.00286324284961,+0.00029414115878,+0.17659375098288,-0.00022061744349,-0.03394658472947,-0.00116664073381},
{+0.00134768152024,+0.00095866771673,-0.00184941931527,+0.00253677762657,-0.00310728060633,-0.00183441394948,-0.00574484305907,-0.18084730217034,-0.00178007584801,-0.00051261274235,-0.02384071090343,-0.00155086936870,-0.00692742658079,+0.20042880757586,-0.00145684835320,-0.00809396671974,+0.04793036630052,-0.00003142342109,-0.00946108421673,-0.00616402348539},
{-0.00012455481643,-0.00614740218005,-0.00129368063005,+0.00030556209525,+0.00187258766006,+0.00161555513441,+0.01307888537417,-0.00692318324910,-0.01380951343412,+0.00367563197549,+0.01076244736608,+0.00315447078976,-0.00866465556771,+0.00767085198640,+0.00229070489522,-0.04565821561589,-0.00037544344854,+0.00842746374754,-0.04568236316136,+0.07582485107892},
{+0.00041834002931,-0.00163524966097,-0.00983084024991,-0.01077651187255,+0.01076843989347,+0.00393925129093,+0.01529745820153,-0.06431810092636,+0.01197741840019,+0.00331868917011,-0.08548604536380,+0.00390618365932,+0.05625672467614,-0.03659929158522,+0.01004763410708,+0.03430765610374,-0.00622100233589,+0.00616769311519,+0.03917990421396,+0.01928164913369},
{-0.02830351331304,+0.24893705002947,-0.03001882944294,-0.18485017003967,-0.00006676259083,+0.00023403446102,-0.00011813133895,-0.00033597110034,+0.00127064253365,+0.00030676705632,-0.00350478686928,+0.00023485636408,-0.00192962536757,-0.00284928701039,-0.00028577595749,+0.00078720576542,+0.00226725161208,+0.00083233879720,-0.00386101960100,+0.00125372601224},
{-0.00017738303432,+0.00715798848651,-0.00143383057771,-0.00230868528322,-0.00001173563718,-0.00084399993381,-0.00295661700793,-0.00078427521007,-0.06660851008748,-0.00201203098355,-0.00297274075512,-0.00116079008281,+0.01063948561162,-0.01488124452064,-0.00029690625119,+0.02524661502938,+0.03024619751119,-0.00294204814919,+0.03120697980676,-0.00510646893120},
{-0.00026909898289,+0.00364296337942,+0.00227027853949,+0.00333650788469,+0.00111429301201,-0.00052351566940,-0.01052815848151,+0.01200067103238,-0.00477050022191,-0.00254116846648,+0.09140224930732,-0.00262189631940,+0.04302313302374,+0.01419635197713,-0.00263366070723,-0.02150319039341,+0.00748135379437,-0.00580455268735,-0.11015619696782,-0.01711586305315},
{+0.00120018816970,-0.00676298468714,-0.00082323000551,+0.00165131907806,-0.00262433073978,+0.00016985639421,-0.00050012286438,-0.04674292565087,+0.00249478507443,+0.00099080033205,+0.03458945123686,+0.00199835191028,-0.01698229046122,-0.07081571342741,+0.00149050066911,+0.13698270926760,+0.00671235311415,+0.00012822313136,-0.05462299338330,+0.01146605284183}};

 
void init_protmats()
{
  long l, m;

  eigmat = (double *) Malloc (20 * sizeof(double));
  for (l = 0; l <= 19; l++)
    {
    if (userrnaSSUM)
      eigmat[l] = rrnaSSUMeigmat[l];
    else if (userrnaSSUB)
      eigmat[l] = rrnaSSUBeigmat[l];
    else if (userrnaSSUE)
      eigmat[l] = rrnaSSUEeigmat[l];
    else if (userrnaLSUE)
      eigmat[l] = rrnaLSUEeigmat[l];

    } 
        
  probmat = (double **) Malloc (20 * sizeof(double *));
  for (l = 0; l < 20; l++)
    probmat[l] = (double *) Malloc (20 * sizeof(double));
  for (l = 0; l <= 19; l++)
    for (m= 0; m <= 19; m++)
      {
      if (userrnaSSUM)
        probmat[l][m] = rrnaSSUMprobmat[l][m];
      else if (userrnaSSUB)
        probmat[l][m] = rrnaSSUBprobmat[l][m];
      else if (userrnaSSUE)
        probmat[l][m] = rrnaSSUEprobmat[l][m];
      else if (userrnaLSUE)
        probmat[l][m] = rrnaLSUEprobmat[l][m];

      }  
}  /* init_protmats */


void getoptions()
{
  /* interactively set options */
  long i, loopcount, loopcount2;
  Char ch;
  boolean didchangecat, didchangercat;
  double probsum;

  fprintf(outfile, "\nAmino acid sequence Maximum Likelihood");
  fprintf(outfile, " method, version %s\n\n",VERSION);
    fprintf(outfile, " rRNA version");

  putchar('\n');
  ctgry = false;
  didchangecat = false;
  rctgry = false;
  didchangercat = false;
  categs = 1;
  rcategs = 1;
  auto_ = false;
  gama = false;
  global = false;
  hypstate = false;
  improve = false;
  invar = false;
  jumble = false;
  njumble = 1;
  lngths = false;
  lambda = 1.0;
  outgrno = 1;
  outgropt = false;
  reconsider = false;
  trout = true;
  usertree = false;
  weights = false;
  printdata = false;
  progress = true;
  treeprint = true;
  userrnaSSUB = true;
  userrnaSSUM = false;
  userrnaSSUE = false;
  userrnaLSUE = false;
  interleaved = true;

  loopcount = 0;
  for (;;){
    cleerhome();
    printf("Amino acid sequence Maximum Likelihood");
    printf(" method, version %s\n\n",VERSION);
    printf("Settings for this run:\n");
    printf("  U                 Search for best tree?  %s\n",
           (usertree ? "No, use user trees in input file" : "Yes"));
    if (usertree) {
      printf("  L          Use lengths from user trees?  %s\n",
               (lngths ? "Yes" : "No"));
    }
    printf("  P   rrnaSSUB, rrnaSSUM or rrnaSSUE or rrnaLSUE  model? ");
           if (userrnaSSUM) printf("%s\n", " rrnaSSUM");
           else if (userrnaSSUB) printf("%s\n", " rrnaSSUB");
           else if (userrnaSSUE) printf("%s\n", " rrnaSSUE");
           else if (userrnaLSUE) printf("%s\n", " rrnaLSUE");

    printf("  C                One category of sites?");
    if (!ctgry || categs == 1)
      printf("  Yes\n");
    else
      printf("  %ld categories of sites\n", categs);
    printf("  R           Rate variation among sites?");
    if (!rctgry)
      printf("  constant rate of change\n");
    else {
      if (gama)
        printf("  Gamma distributed rates\n");
      else {
        if (invar)
          printf("  Gamma+Invariant sites\n");
        else
          printf("  user-defined HMM of rates\n");
      }
      printf("  A   Rates at adjacent sites correlated?");
      if (!auto_)
        printf("  No, they are independent\n");
      else
        printf("  Yes, mean block length =%6.1f\n", 1.0 / lambda);
    }
    printf("  W                       Sites weighted?  %s\n",
           (weights ? "Yes" : "No"));
    if ((!usertree) || reconsider) {
      printf("  S        Speedier but rougher analysis?  %s\n",
             (improve ? "No, not rough" : "Yes"));
      printf("  G                Global rearrangements?  %s\n",
             (global ? "Yes" : "No"));
    }
    if (!usertree) {
      printf("  J   Randomize input order of sequences?");
      if (jumble)
        printf("  Yes (seed =%8ld,%3ld times)\n", inseed0, njumble);
      else
        printf("  No. Use input order\n");
    }
    else
       printf("  V    Rearrange starting with user tree?  %s\n",
             (reconsider ? "Yes" : "No"));
    printf("  O                        Outgroup root?  %s%3ld\n",
           (outgropt ? "Yes, at sequence number" :
                       "No, use as outgroup species"),outgrno);
    printf("  M           Analyze multiple data sets?");
    if (mulsets)
      printf("  Yes, %2ld %s\n", datasets,
               (justwts ? "sets of weights" : "data sets"));
    else
      printf("  No\n");
    printf("  I          Input sequences interleaved?  %s\n",
           (interleaved ? "Yes" : "No, sequential"));
    printf("  0   Terminal type (IBM PC, ANSI, none)?  %s\n",
           (ibmpc ? "IBM PC" : ansi  ? "ANSI" : "(none)"));
    printf("  1    Print out the data at start of run  %s\n",
           (printdata ? "Yes" : "No"));
    printf("  2  Print indications of progress of run  %s\n",
           (progress ? "Yes" : "No"));
    printf("  3                        Print out tree  %s\n",
           (treeprint ? "Yes" : "No"));
    printf("  4       Write out trees onto tree file?  %s\n",
           (trout ? "Yes" : "No"));
    printf("  5   Reconstruct hypothetical sequences?  %s\n",
           (hypstate ? "Yes" : "No"));
    printf("\n  Y to accept these or type the letter for one to change\n");
#ifdef WIN32
    phyFillScreenColor();
#endif
    scanf("%c%*[^\n]", &ch);
    getchar();
    if (ch == '\n')
      ch = ' ';
    uppercase(&ch);
    if (ch == 'Y')
      break;
    if (strchr("UPLCRAWSGJVOMI012345",ch) != NULL){
      switch (ch) {

      case 'C':
        ctgry = !ctgry;
        if (ctgry) {
          printf("\nSitewise user-assigned categories:\n\n");
          initcatn(&categs);
          if (rate){
            free(rate);
          }
          rate    = (double *) Malloc(categs * sizeof(double));
          didchangecat = true;
          initcategs(categs, rate);
        }
        break;

      case 'P':
        if (userrnaSSUB) {
          userrnaSSUB = false;
          userrnaSSUM = true; 
          userrnaSSUE = false; 
          userrnaLSUE = false; }
        else 
         if (userrnaSSUM)  {
            userrnaSSUB = false;
            userrnaSSUM = false;
            userrnaSSUE= true;    
            userrnaLSUE = false; }
        else
         if (userrnaSSUE){
            userrnaSSUB = false;
            userrnaSSUM = false;
            userrnaSSUE= false;  
            userrnaLSUE = true; }
         else
         if (userrnaLSUE){
            userrnaSSUB = true;
            userrnaSSUM = false;
            userrnaSSUE= false;   
            userrnaLSUE = false; }
            
        break;

      case 'R':
        if (!rctgry) {
          rctgry = true;
          gama = true;
        } else {
          if (gama) {
            gama = false;
            invar = true;
          } else {
            if (invar)
              invar = false;
            else
              rctgry = false;
          }
        }
        break;

      case 'A':
        auto_ = !auto_;
        if (auto_)
          initlambda(&lambda);
        break;

      case 'W':
        weights = !weights;
        break;

      case 'S':
        improve = !improve;
        break;

      case 'G':
        global = !global;
        break;

      case 'J':
        jumble = !jumble;
        if (jumble)
          initjumble(&inseed, &inseed0, seed, &njumble);
        else njumble = 1;
        break;

      case 'L':
        lngths = !lngths;
        break;

      case 'O':
        outgropt = !outgropt;
        if (outgropt)
          initoutgroup(&outgrno, spp);
        break;

      case 'U':
        usertree = !usertree;
        break;

      case 'V':
        reconsider = !reconsider;
        break;

      case 'M':
        mulsets = !mulsets;
        if (mulsets) {
          printf("Multiple data sets or multiple weights?");
          loopcount2 = 0;
          do {
            printf(" (type D or W)\n");
#ifdef WIN32
            phyFillScreenColor();
#endif
            scanf("%c%*[^\n]", &ch2);
            getchar();
            if (ch2 == '\n')
                ch2 = ' ';
            uppercase(&ch2);
            countup(&loopcount2, 10);
          } while ((ch2 != 'W') && (ch2 != 'D'));
          justwts = (ch2 == 'W');
          if (justwts)
            justweights(&datasets);
          else
            initdatasets(&datasets);
          if (!jumble) {
            jumble = true;
            initjumble(&inseed, &inseed0, seed, &njumble);
          }
        }
        break;

      case 'I':
        interleaved = !interleaved;
        break;

      case '0':
        initterminal(&ibmpc, &ansi);
        break;

      case '1':
        printdata = !printdata;
        break;

      case '2':
        progress = !progress;
        break;

      case '3':
        treeprint = !treeprint;
        break;

      case '4':
        trout = !trout;
        break;

      case '5':
        hypstate = !hypstate;
        break;
      }
    } else
      printf("Not a possible option!\n");
    countup(&loopcount, 100);
  }
  if (gama || invar) {
    loopcount = 0;
    do {
      printf(
"\nCoefficient of variation of substitution rate among sites (must be positive)\n");
      printf(
  " In gamma distribution parameters, this is 1/(square root of alpha)\n");
#ifdef WIN32
      phyFillScreenColor();
#endif
      scanf("%lf%*[^\n]", &cv);
      getchar();
      countup(&loopcount, 10);
    } while (cv <= 0.0);
    alpha = 1.0 / (cv * cv);
  }
  if (!rctgry)
    auto_ = false;
  if (rctgry) {
    printf("\nRates in HMM");
    if (invar)
      printf(" (including one for invariant sites)");
    printf(":\n");
    initcatn(&rcategs);
    if (probcat){
      free(probcat);
      free(rrate);
    }
    probcat = (double *) Malloc(rcategs * sizeof(double));
    rrate   = (double *) Malloc(rcategs * sizeof(double));
    didchangercat = true;
    if (gama)
      initgammacat(rcategs, alpha, rrate, probcat);
    else {
      if (invar) {
        loopcount = 0;
        do {
          printf("Fraction of invariant sites?\n");
          scanf("%lf%*[^\n]", &invarfrac);
          getchar();
          countup (&loopcount, 10);
        } while ((invarfrac <= 0.0) || (invarfrac >= 1.0));
        initgammacat(rcategs-1, alpha, rrate, probcat);
        for (i = 0; i < rcategs-1; i++)
          probcat[i] = probcat[i]*(1.0-invarfrac);
        probcat[rcategs-1] = invarfrac;
        rrate[rcategs-1] = 0.0;
      } else {
        initcategs(rcategs, rrate);
        initprobcat(rcategs, &probsum, probcat);
      }
    }
  }
  if (!didchangercat){
    rrate      = (double *) Malloc(rcategs*sizeof(double));
    probcat    = (double *) Malloc(rcategs*sizeof(double));
    rrate[0]   = 1.0;
    probcat[0] = 1.0;
  }
  if (!didchangecat) {
    rate       = (double *) Malloc(categs*sizeof(double));
    rate[0]    = 1.0;
  }
  init_protmats(); 

}  /* getoptions */


void makeprotfreqs()
{
  /* calculate amino acid frequencies based on eigmat */
  long i, mineig;

  mineig = 0;
  for (i = 0; i <= 19; i++)
    if (fabs(eigmat[i]) < fabs(eigmat[mineig]))
      mineig = i;
  memcpy(freqaa, probmat[mineig], 20 * sizeof(double));
  for (i = 0; i <= 19; i++)
    freqaa[i] = fabs(freqaa[i]);
} /* makeprotfreqs */


void allocrest()
{
  long i;

  y = (Char **) Malloc(spp*sizeof(Char *));
  for (i = 0; i < spp; i++)
    y[i] = (Char *) Malloc(sites*sizeof(Char));
  nayme       = (naym *) Malloc(spp*sizeof(naym));
  enterorder  = (long *) Malloc(spp*sizeof(long));
  category    = (long *) Malloc(sites*sizeof(long));
  weight      = (long *) Malloc(sites*sizeof(long));
  alias       = (long *) Malloc(sites*sizeof(long));
  ally        = (long *) Malloc(sites*sizeof(long));
  location    = (long *) Malloc(sites*sizeof(long));
  aliasweight = (long *) Malloc(sites*sizeof(long));
}  /* allocrest */


void doinit()
{ /* initializes variables */
  inputnumbers(&spp, &sites, &nonodes2, 2);
  getoptions();
  makeprotfreqs();
  if (printdata)
    fprintf(outfile, "%2ld species, %3ld  sites\n", spp, sites);
  alloctree(&curtree.nodep, nonodes2, usertree);
  allocrest();
  if (usertree && !reconsider)
    return;
  alloctree(&bestree.nodep, nonodes2, 0);
  alloctree(&priortree.nodep, nonodes2, 0);
  if (njumble <= 1)
    return;
  alloctree(&bestree2.nodep, nonodes2, 0);
}  /* doinit */


void inputoptions()
{
  long i;

  if (!firstset)
    samenumsp(&sites, ith);
  if (firstset) {
    for (i = 0; i < sites; i++)
      category[i] = 1;
    for (i = 0; i < sites; i++)
      weight[i] = 1;
  }
  if (justwts || weights)
    inputweights(sites, weight, &weights);
  weightsum = 0;
  for (i = 0; i < sites; i++)
    weightsum += weight[i];
  if ((ctgry && categs > 1) && (firstset || !justwts)) {
    inputcategs(0, sites, category, categs, "ProML");
    if (printdata)
      printcategs(outfile, sites, category, "Site categories");
  }
  if (weights && printdata)
    printweights(outfile, 0, sites, weight, "Sites");
      if (userrnaSSUB)     fprintf(outfile, "\n\n rrnaSSUB");
      else if (userrnaSSUM) fprintf(outfile, "\n\n rrnaSSUM");
      else if (userrnaSSUE)  fprintf(outfile, "\n\n rrnaSSUE");
      else if (userrnaLSUE)  fprintf(outfile, "\n\n rrnaLSUE");

   fprintf(outfile, " model of rRNA symbol change") ;
     
}  /* inputoptions */


void input_protdata(long chars)
{
  /* input the names and sequences for each species */
  /* used by proml */
  long i, j, k, l, basesread, basesnew;
  Char charstate;
  boolean allread, done;

  if (printdata)
    headings(chars, "Sequences", "---------");
  basesread = 0;
  basesnew = 0;
  allread = false;
  while (!(allread)) {
    allread = true;
    if (eoln(infile))
      scan_eoln(infile);
    i = 1;
    while (i <= spp) {
      if ((interleaved && basesread == 0) || !interleaved)
        initname(i - 1);
      j = (interleaved) ? basesread : 0;
      done = false;
      while (!done && !eoff(infile)) {
        if (interleaved)
          done = true;
        while (j < chars && !(eoln(infile) || eoff(infile))) {
          charstate = gettc(infile);
          if (charstate == '\n')
            charstate = ' ';
          if (charstate == ' ' || (charstate >= '0' && charstate <= '9'))
            continue;
          uppercase(&charstate);
          if ((strchr("ABCDEFGHIKLMNPQRSTVWXYZ*?-", charstate)) == NULL) {
        printf("ERROR: bad amino acid: %c at position %ld of species %ld\n",
                   charstate, j+1, i);
            if (charstate == '.') {
          printf("       Periods (.) may not be used as gap characters.\n");
          printf("       The correct gap character is (-)\n");
            }
            exxit(-1);
          }
          j++;
          y[i - 1][j - 1] = charstate;
        }
        if (interleaved)
          continue;
        if (j < chars)
          scan_eoln(infile);
        else if (j == chars)
          done = true;
      }
      if (interleaved && i == 1)
        basesnew = j;

      scan_eoln(infile);

      if ((interleaved && j != basesnew) ||
          (!interleaved && j != chars)) {
        printf("ERROR: SEQUENCES OUT OF ALIGNMENT AT POSITION %ld.\n", j);
        exxit(-1);
      }
      i++;
    }

    if (interleaved) {
      basesread = basesnew;
      allread = (basesread == chars);
    } else
      allread = (i > spp);
  }
  if (!printdata)
    return;
  for (i = 1; i <= ((chars - 1) / 60 + 1); i++) {
    for (j = 1; j <= spp; j++) {
      for (k = 0; k < nmlngth; k++)
        putc(nayme[j - 1][k], outfile);
      fprintf(outfile, "   ");
      l = i * 60;
      if (l > chars)
        l = chars;
      for (k = (i - 1) * 60 + 1; k <= l; k++) {
        if (j > 1 && y[j - 1][k - 1] == y[0][k - 1])
          charstate = '.';
        else
          charstate = y[j - 1][k - 1];
        putc(charstate, outfile);
        if (k % 10 == 0 && k % 60 != 0)
          putc(' ', outfile);
      }
      putc('\n', outfile);
    }
    putc('\n', outfile);
  }
  putc('\n', outfile);
}  /* input_protdata */


void makeweights()
{
  /* make up weights vector to avoid duplicate computations */
  long i;

  for (i = 1; i <= sites; i++) {
    alias[i - 1] = i;
    ally[i - 1] = 0;
    aliasweight[i - 1] = weight[i - 1];
    location[i - 1] = 0;
  }
  sitesort2   (sites, aliasweight);
  sitecombine2(sites, aliasweight);
  sitescrunch2(sites, 1, 2, aliasweight);
  for (i = 1; i <= sites; i++) {
    if (aliasweight[i - 1] > 0)
      endsite = i;
  }
  for (i = 1; i <= endsite; i++) {
    location[alias[i - 1] - 1] = i;
    ally[alias[i - 1] - 1] = alias[i - 1];
  }
  term = (double **) Malloc(endsite * sizeof(double *));
  for (i = 0; i < endsite; i++)
     term[i] = (double *) Malloc(rcategs * sizeof(double));
  slopeterm = (double **) Malloc(endsite * sizeof(double *));
  for (i = 0; i < endsite; i++)
     slopeterm[i] = (double *) Malloc(rcategs * sizeof(double));
  curveterm = (double **) Malloc(endsite * sizeof(double *));
  for (i = 0; i < endsite; i++)
     curveterm[i] = (double *) Malloc(rcategs * sizeof(double));
  mp = (vall *) Malloc(sites*sizeof(vall));
  contribution = (contribarr *) Malloc(endsite*sizeof(contribarr));
}  /* makeweights */


void prot_makevalues(long categs, pointarray treenode, long endsite,
                        long spp, sequence y, steptr alias)
{
  /* set up fractional likelihoods at tips   */
  /* a version of makevalues2 found in seq.c */
  /* used by proml                             */
  long i, j, k, l;
  long b;

  for (k = 0; k < endsite; k++) {
    j = alias[k];
    for (i = 0; i < spp; i++) {
      for (l = 0; l < categs; l++) {
        memset(treenode[i]->protx[k][l], 0, sizeof(double)*20);
        switch (y[i][j - 1]) {

        case 'A':
          treenode[i]->protx[k][l][0] = 1.0;
          break;

        case 'R':
          treenode[i]->protx[k][l][(long)arginine   - (long)alanine] = 1.0;
          break;

        case 'N':
          treenode[i]->protx[k][l][(long)asparagine - (long)alanine] = 1.0;
          break;

        case 'D':
          treenode[i]->protx[k][l][(long)aspartic   - (long)alanine] = 1.0;
          break;

        case 'C':
          treenode[i]->protx[k][l][(long)cysteine   - (long)alanine] = 1.0;
          break;

        case 'Q':
          treenode[i]->protx[k][l][(long)glutamine  - (long)alanine] = 1.0;
          break;

        case 'E':
          treenode[i]->protx[k][l][(long)glutamic   - (long)alanine] = 1.0;
          break;

        case 'G':
          treenode[i]->protx[k][l][(long)glycine    - (long)alanine] = 1.0;
          break;

        case 'H':
          treenode[i]->protx[k][l][(long)histidine  - (long)alanine] = 1.0;
          break;

        case 'I':
          treenode[i]->protx[k][l][(long)isoleucine - (long)alanine] = 1.0;
          break;

        case 'L':
          treenode[i]->protx[k][l][(long)leucine    - (long)alanine] = 1.0;
          break;

        case 'K':
          treenode[i]->protx[k][l][(long)lysine     - (long)alanine] = 1.0;
          break;

        case 'M':
          treenode[i]->protx[k][l][(long)methionine - (long)alanine] = 1.0;
          break;

        case 'F':
          treenode[i]->protx[k][l][(long)phenylalanine - (long)alanine] = 1.0;
          break;

        case 'P':
          treenode[i]->protx[k][l][(long)proline    - (long)alanine] = 1.0;
          break;

        case 'S':
          treenode[i]->protx[k][l][(long)serine     - (long)alanine] = 1.0;
          break;

        case 'T':
          treenode[i]->protx[k][l][(long)threonine  - (long)alanine] = 1.0;
          break;

        case 'W':
          treenode[i]->protx[k][l][(long)tryptophan - (long)alanine] = 1.0;
          break;

        case 'Y':
          treenode[i]->protx[k][l][(long)tyrosine   - (long)alanine] = 1.0;
          break;

        case 'V':
          treenode[i]->protx[k][l][(long)valine     - (long)alanine] = 1.0;
          break;

        case 'B':
          treenode[i]->protx[k][l][(long)asparagine - (long)alanine] = 1.0;
          treenode[i]->protx[k][l][(long)aspartic   - (long)alanine] = 1.0;
          break;

        case 'Z':
          treenode[i]->protx[k][l][(long)glutamine  - (long)alanine] = 1.0;
          treenode[i]->protx[k][l][(long)glutamic   - (long)alanine] = 1.0;
          break;

        case 'X':                /* unknown aa                            */
          for (b = 0; b <= 19; b++)
            treenode[i]->protx[k][l][b] = 1.0;
          break;

        case '?':                /* unknown aa                            */
          for (b = 0; b <= 19; b++)
            treenode[i]->protx[k][l][b] = 1.0;
          break;

        case '*':                /* stop codon symbol                    */
          for (b = 0; b <= 19; b++)
            treenode[i]->protx[k][l][b] = 1.0;
          break;

        case '-':                /* deletion event-absent data or aa */
          for (b = 0; b <= 19; b++)
            treenode[i]->protx[k][l][b] = 1.0;
          break;
        }
      }
    }
  }
}  /* prot_makevalues */


void alloc_pmatrix(long sib)
{
  /* Allocate memory for a new pmatrix.  Called iff num_sibs>max_num_sibs */
  long j, k, l;
  double ****temp_matrix;

  temp_matrix = (double ****) Malloc (rcategs * sizeof(double ***));
  for (j = 0; j < rcategs; j++) {
    temp_matrix[j] = (double ***) Malloc(categs * sizeof(double **));
    for (k = 0; k < categs; k++) {
      temp_matrix[j][k] = (double **) Malloc(20 * sizeof (double *));
      for (l = 0; l < 20; l++)
        temp_matrix[j][k][l] = (double *) Malloc(20 * sizeof(double));
    }
  }
  pmatrices[sib] = temp_matrix;
  max_num_sibs++;
}  /* alloc_pmatrix */


void prot_inittable()
{
  /* Define a lookup table. Precompute values and print them out in tables */
  /* Allocate memory for the pmatrices, dpmatices and ddpmatrices          */
  long i, j, k, l;
  double sumrates;

  /* Allocate memory for pmatrices, the array of pointers to pmatrices     */

  pmatrices = (double *****) Malloc ( spp * sizeof(double ****));

  /* Allocate memory for the first 2 pmatrices, the matrix of conversion   */
  /* probabilities, but only once per run (aka not on the second jumble.   */

    alloc_pmatrix(0);
    alloc_pmatrix(1);

  /*  Allocate memory for one dpmatrix, the first derivative matrix        */

  dpmatrix = (double ****) Malloc( rcategs * sizeof(double ***));
  for (j = 0; j < rcategs; j++) {
    dpmatrix[j] = (double ***) Malloc( categs * sizeof(double **));
    for (k = 0; k < categs; k++) {
      dpmatrix[j][k] = (double **) Malloc( 20 * sizeof(double *));
      for (l = 0; l < 20; l++)
        dpmatrix[j][k][l] = (double *) Malloc( 20 * sizeof(double));
    }
  }

  /*  Allocate memory for one ddpmatrix, the second derivative matrix      */
  ddpmatrix = (double ****) Malloc( rcategs * sizeof(double ***));
  for (j = 0; j < rcategs; j++) {
    ddpmatrix[j] = (double ***) Malloc( categs * sizeof(double **));
    for (k = 0; k < categs; k++) {
      ddpmatrix[j][k] = (double **) Malloc( 20 * sizeof(double *));
      for (l = 0; l < 20; l++)
        ddpmatrix[j][k][l] = (double *) Malloc( 20 * sizeof(double));
    }
  }

  /* Allocate memory and assign values to tbl, the matrix of possible rates*/

  tbl = (double **) Malloc( rcategs * sizeof(double *));
  for (j = 0; j < rcategs; j++)
    tbl[j] = (double *) Malloc( categs * sizeof(double));

  for (j = 0; j < rcategs; j++)
    for (k = 0; k < categs; k++)
      tbl[j][k] = rrate[j]*rate[k];

  sumrates = 0.0;
  for (i = 0; i < endsite; i++) {
    for (j = 0; j < rcategs; j++)
      sumrates += aliasweight[i] * probcat[j]
        * tbl[j][category[alias[i] - 1] - 1];
  }
  sumrates /= (double)sites;
  for (j = 0; j < rcategs; j++)
    for (k = 0; k < categs; k++) {
      tbl[j][k] /= sumrates;
    }
  if (gama) {
    fprintf(outfile, "\nDiscrete approximation to gamma distributed rates\n");
    fprintf(outfile,
    " Coefficient of variation of rates = %f  (alpha = %f)\n",
      cv, alpha);
  }
  if (rcategs > 1) {
    fprintf(outfile, "\nStates in HMM   Rate of change    Probability\n\n");
    for (i = 0; i < rcategs; i++)
      if (probcat[i] < 0.0001)
        fprintf(outfile, "%9ld%16.3f%20.6f\n", i+1, rrate[i], probcat[i]);
      else if (probcat[i] < 0.001)
          fprintf(outfile, "%9ld%16.3f%19.5f\n", i+1, rrate[i], probcat[i]);
        else if (probcat[i] < 0.01)
            fprintf(outfile, "%9ld%16.3f%18.4f\n", i+1, rrate[i], probcat[i]);
          else
            fprintf(outfile, "%9ld%16.3f%17.3f\n", i+1, rrate[i], probcat[i]);
    putc('\n', outfile);
    if (auto_)
      fprintf(outfile,
     "Expected length of a patch of sites having the same rate = %8.3f\n",
             1/lambda);
    putc('\n', outfile);
  }
  if (categs > 1) {
    fprintf(outfile, "\nSite category   Rate of change\n\n");
    for (k = 0; k < categs; k++)
      fprintf(outfile, "%9ld%16.3f\n", k+1, rate[k]);
  }
  if ((rcategs  > 1) || (categs >> 1))
    fprintf(outfile, "\n\n");
}  /* prot_inittable */


void getinput()
{
  /* reads the input data */
  if (!justwts || firstset)
    inputoptions();
  if (!justwts || firstset)
    input_protdata(sites);
  makeweights();
  setuptree2(curtree);
  if (!usertree || reconsider) {
    setuptree2(bestree);
    setuptree2(priortree);
    if (njumble > 1)
      setuptree2(bestree2);
  }
  prot_allocx(nonodes2, rcategs, curtree.nodep, usertree);
  if (!usertree || reconsider) {
    prot_allocx(nonodes2, rcategs, bestree.nodep, 0);
    prot_allocx(nonodes2, rcategs, priortree.nodep, 0);
    if (njumble > 1)
      prot_allocx(nonodes2, rcategs, bestree2.nodep, 0);
  }
  prot_makevalues(rcategs, curtree.nodep, endsite, spp, y, alias);
}  /* getinput */


void inittravtree(node *p)
{
  /* traverse tree to set initialized and v to initial values */

  p->initialized = false;
  p->back->initialized = false;
  if (!p->tip) {
    inittravtree(p->next->back);
    inittravtree(p->next->next->back);
  }
}  /* inittravtree */


void prot_nuview(node *p)
{
  long i, j, k, l, m, num_sibs, sib_index;
  node *sib_ptr, *sib_back_ptr;
  psitelike prot_xx, x2;
  double lw, prod7;
  double **pmat;

  /* Figure out how many siblings the current node has  */
  /* and be sure that pmatrices is large enough         */
  num_sibs = count_sibs(p);
  for (i = 0; i < num_sibs; i++)
    if (pmatrices[i] == NULL)
      alloc_pmatrix(i);

  /* Recursive calls, should be called for all children */
  sib_ptr = p;
  for (i=0 ; i < num_sibs; i++) {
    sib_ptr      = sib_ptr->next;
    sib_back_ptr = sib_ptr->back;

    if (!sib_back_ptr->tip &&
        !sib_back_ptr->initialized)
      prot_nuview(sib_back_ptr);
  }

  /* Make pmatrices for all possible combinations of category, rcateg      */
  /* and sib                                                                   */
  sib_ptr = p;                                /* return to p */
  for (sib_index=0; sib_index < num_sibs; sib_index++) {
    sib_ptr      = sib_ptr->next;
    sib_back_ptr = sib_ptr->back;

    lw = sib_back_ptr->v;

    for (j = 0; j < rcategs; j++)
      for (k = 0; k < categs; k++)
        make_pmatrix(pmatrices[sib_index][j][k], NULL, NULL, 0, lw,
                                        tbl[j][k], eigmat, probmat);
  }

  for (i = 0; i < endsite; i++) {
    k = category[alias[i]-1] - 1;
    for (j = 0; j < rcategs; j++) {

      /* initialize to 1 all values of prot_xx */
      for (m = 0; m <= 19; m++)
        prot_xx[m] = 1;

      sib_ptr = p;                        /* return to p */
      /* loop through all sibs and calculate likelihoods for all possible*/
      /* amino acid combinations                                         */
      for (sib_index=0; sib_index < num_sibs; sib_index++) {
        sib_ptr      = sib_ptr->next;
        sib_back_ptr = sib_ptr->back;

        memcpy(x2, sib_back_ptr->protx[i][j], sizeof(psitelike));
        pmat = pmatrices[sib_index][j][k];
        for (m = 0; m <= 19; m++) {
          prod7 = 0;
          for (l = 0; l <= 19; l++)
            prod7 += (pmat[m][l] * x2[l]);
          prot_xx[m] *= prod7;
        }
      }
      /* And the final point of this whole function: */
      memcpy(p->protx[i][j], prot_xx, sizeof(psitelike));
    }
  }

  p->initialized = true;
}  /* prot_nuview */


void prot_slopecurv(node *p,double y,double *like,double *slope,double *curve)
{
  /* compute log likelihood, slope and curvature at node p */
  long i, j, k, l, m, lai;
  double sum, sumc, sumterm, lterm, sumcs, sumcc, sum2, slope2, curve2;
  double frexm = 0;                        /* frexm = freqaa[m]*x1[m] */
                                        /* frexml = frexm*x2[l]    */
  double prod4m, prod5m, prod6m;        /* elements of prod4-5 for */
                                        /* each m                   */
  double **pmat, **dpmat, **ddpmat;        /* local pointers to global*/
                                        /* matrices                   */
  double prod4, prod5, prod6;
  contribarr thelike, nulike, nuslope, nucurve,
    theslope, thecurve, clai, cslai, cclai;
  node *q;
  psitelike x1, x2;

  q = p->back;
  sum = 0.0;
  for (j = 0; j < rcategs; j++) {
    for (k = 0; k < categs; k++) {
      make_pmatrix(pmatrices[0][j][k], dpmatrix[j][k], ddpmatrix[j][k],
                                        2, y, tbl[j][k], eigmat, probmat);
    }
  }
  for (i = 0; i < endsite; i++) {
    k = category[alias[i]-1] - 1;
    for (j = 0; j < rcategs; j++) {
      memcpy(x1, p->protx[i][j], sizeof(psitelike));
      memcpy(x2, q->protx[i][j], sizeof(psitelike));
      pmat = pmatrices[0][j][k];
      dpmat = dpmatrix[j][k];
      ddpmat = ddpmatrix[j][k];
      prod4 = 0.0;
      prod5 = 0.0;
      prod6 = 0.0;
      for (m = 0; m <= 19; m++) {
        prod4m = 0.0;
        prod5m = 0.0;
        prod6m = 0.0;
        frexm = x1[m] * freqaa[m];
        for (l = 0; l <= 19; l++) {
          prod4m += x2[l] * pmat[m][l];
          prod5m += x2[l] * dpmat[m][l];
          prod6m += x2[l] * ddpmat[m][l];
        }
        prod4 += frexm * prod4m;
        prod5 += frexm * prod5m;
        prod6 += frexm * prod6m;
      }
      term[i][j] = prod4;
      slopeterm[i][j] = prod5;
      curveterm[i][j] = prod6;
    }
    sumterm = 0.0;
    for (j = 0; j < rcategs; j++)
      sumterm += probcat[j] * term[i][j];
    if (sumterm < 0.0)
        sumterm = 0.00000001;        /* ??? */
    lterm = log(sumterm);
    for (j = 0; j < rcategs; j++) {
      term[i][j] = term[i][j] / sumterm;
      slopeterm[i][j] = slopeterm[i][j] / sumterm;
      curveterm[i][j] = curveterm[i][j] / sumterm;
    }
    sum += (aliasweight[i] * lterm);
  }
  for (i = 0; i < rcategs; i++) {
    thelike[i] = 1.0;
    theslope[i] = 0.0;
    thecurve[i] = 0.0;
  }
  for (i = 0; i < sites; i++) {
    sumc = 0.0;
    sumcs = 0.0;
    sumcc = 0.0;
    for (k = 0; k < rcategs; k++) {
      sumc += probcat[k] * thelike[k];
      sumcs += probcat[k] * theslope[k];
      sumcc += probcat[k] * thecurve[k];
    }
    sumc *= lambda;
    sumcs *= lambda;
    sumcc *= lambda;
    if ((ally[i] > 0) && (location[ally[i]-1] > 0)) {
      lai = location[ally[i] - 1];
      memcpy(clai, term[lai - 1], rcategs*sizeof(double));
      memcpy(cslai, slopeterm[lai - 1], rcategs*sizeof(double));
      memcpy(cclai, curveterm[lai - 1], rcategs*sizeof(double));
      if (weight[i] > 1) {
        for (j = 0; j < rcategs; j++) {
          if (clai[j] > 0.0)
            clai[j] = exp(weight[i]*log(clai[j]));
          else clai[j] = 0.0;
          if (cslai[j] > 0.0)
            cslai[j] = exp(weight[i]*log(cslai[j]));
          else cslai[j] = 0.0;
          if (cclai[j] > 0.0)
            cclai[j] = exp(weight[i]*log(cclai[j]));
          else cclai[j] = 0.0;
        }
      }
      for (j = 0; j < rcategs; j++) {
        nulike[j]  = ((1.0 - lambda) * thelike[j]  + sumc) *  clai[j];
        nuslope[j] = ((1.0 - lambda) * theslope[j] + sumcs) * clai[j]
                   + ((1.0 - lambda) * thelike[j]  + sumc) *  cslai[j];
        nucurve[j] = ((1.0 - lambda) * thecurve[j] + sumcc) * clai[j]
             + 2.0 * ((1.0 - lambda) * theslope[j] + sumcs) * cslai[j]
                   + ((1.0 - lambda) * thelike[j]  + sumc) *  cclai[j];
      }
    } else {
      for (j = 0; j < rcategs; j++) {
        nulike[j]  = ((1.0 - lambda) * thelike[j]  + sumc);
        nuslope[j] = ((1.0 - lambda) * theslope[j] + sumcs);
        nucurve[j] = ((1.0 - lambda) * thecurve[j] + sumcc);
      }
    }
    memcpy(thelike, nulike, rcategs*sizeof(double));
    memcpy(theslope, nuslope, rcategs*sizeof(double));
    memcpy(thecurve, nucurve, rcategs*sizeof(double));
  }
  sum2 = 0.0;
  slope2 = 0.0;
  curve2 = 0.0;
  for (i = 0; i < rcategs; i++) {
    sum2 += probcat[i] * thelike[i];
    slope2 += probcat[i] * theslope[i];
    curve2 += probcat[i] * thecurve[i];
  }
  sum += log(sum2);
  (*like) = sum;
  (*slope) = slope2 / sum2;
  (*curve) = (curve2 - slope2 * slope2 / sum2) / sum2;
} /* prot_slopecurv */


void makenewv(node *p)
{
  /* Newton-Raphson algorithm improvement of a branch length */
  long it, ite;
  double y, yold=0, yorig, like, slope, curve, oldlike=0;
  boolean done, firsttime, better;
  node *q;

  q = p->back;
  y = p->v;
  yorig = y;
  done = false;
  firsttime = true;
  it = 1;
  ite = 0;
  while ((it < iterations) && (ite < 20) && (!done)) {
    prot_slopecurv(p, y, &like, &slope, &curve);
    better = false;
    if (firsttime) {
      yold = y;
      oldlike = like;
      firsttime = false;
      better = true;
    } else {
      if (like > oldlike) {
        yold = y;
        oldlike = like;
        better = true;
        it++;
      }
    }
    if (better) {
      y = y + slope/fabs(curve);
      if (y < epsilon)
        y = epsilon;
    } else {
      if (fabs(y - yold) < epsilon)
        ite = 20;
      y = (y + (7 * yold)) / 8;
    }
    ite++;
    done = fabs(y-yold) < epsilon;
  }
  smoothed = (fabs(yold-yorig) < epsilon) && (yorig > 1000.0*epsilon);
  p->v = yold;
  q->v = yold;
  curtree.likelihood = oldlike;
}  /* makenewv */


void update(node *p)
{
  long num_sibs, i;
  node *sib_ptr;

  if (!p->tip && !p->initialized)
    prot_nuview(p);
  if (!p->back->tip && !p->back->initialized)
    prot_nuview(p->back);
  if (p->iter) {
    makenewv(p);

    if (!p->tip) {
      num_sibs = count_sibs(p);
      sib_ptr  = p;
      for (i=0; i < num_sibs; i++) {
        sib_ptr              = sib_ptr->next;
        sib_ptr->initialized = false;
      }
    }

    if (!p->back->tip) {
      num_sibs = count_sibs(p->back);
      sib_ptr  = p->back;
      for (i=0; i < num_sibs; i++) {
        sib_ptr              = sib_ptr->next;
        sib_ptr->initialized = false;
      }
    }
  }
}  /* update */


void smooth(node *p)
{
  long i, num_sibs;
  node *sib_ptr;

  smoothed = false;
  update(p);
  if (p->tip)
    return;

  num_sibs = count_sibs(p);
  sib_ptr  = p;

  for (i=0; i < num_sibs; i++) {
    sib_ptr = sib_ptr->next;

    if (polishing || (smoothit && !smoothed)) {
      smooth(sib_ptr->back);
      p->initialized = false;
      sib_ptr->initialized = false;
    }
  }
}  /* smooth */


void make_pmatrix(double **matrix, double **dmat, double **ddmat,
                        long derivative, double lz, double rat,
                        double *eigmat, double **probmat)
{
  /* Computes the R matrix such that matrix[m][l] is the joint probability */
  /* of m and l.                                                           */
  /* Computes a P matrix such that matrix[m][l] is the conditional         */
  /* probability of m given l.  This is accomplished by dividing all terms */
  /* in the R matrix by freqaa[m], the frequency of l.                     */

  long k, l, m;                 /* (l) original character state */
                                /* (m) final    character state */
                                /* (k) lambda counter           */
  double p0, p1, p2, q;
  double elambdat[20], delambdat[20], ddelambdat[20];
                                /* exponential term for matrix  */
                                /* and both derivative matrices */

  for (k = 0; k <= 19; k++) {
    elambdat[k] = exp(lz * rat * eigmat[k]);
    if(derivative != 0) {
        delambdat[k] = (elambdat[k] * rat * eigmat[k]);
        ddelambdat[k] = (delambdat[k] * rat * eigmat[k]);
    }
   }
  for (m = 0; m <= 19; m++) {
    for (l = 0; l <= 19; l++) {
      p0 = 0.0;
      p1 = 0.0;
      p2 = 0.0;
      for (k = 0; k <= 19; k++) {
        q = probmat[k][m] * probmat[k][l];
        p0 += (q * elambdat[k]);
        if(derivative !=0) {
          p1 += (q * delambdat[k]);
          p2 += (q * ddelambdat[k]);
        }
      }
      matrix[m][l] = p0 / freqaa[m];
      if(derivative != 0) {
        dmat[m][l] = p1 / freqaa[m];
        ddmat[m][l] = p2 / freqaa[m];
      }
    }
  }
}  /* make_pmatrix */


double prot_evaluate(node *p, boolean saveit)
{
  contribarr tterm;
  double sum, sum2, sumc, y, prod4, prodl, frexm, sumterm, lterm;
  double **pmat;
  long i, j, k, l, m, lai;
  node *q;
  psitelike x1, x2;

  sum = 0.0;
  q = p->back;
  y = p->v;
  for (j = 0; j < rcategs; j++)
    for (k = 0; k < categs; k++)
      make_pmatrix(pmatrices[0][j][k],NULL,NULL,0,y,tbl[j][k],eigmat,probmat);
  for (i = 0; i < endsite; i++) {
    k = category[alias[i]-1] - 1;
    for (j = 0; j < rcategs; j++) {
      memcpy(x1, p->protx[i][j], sizeof(psitelike));
      memcpy(x2, q->protx[i][j], sizeof(psitelike));
      prod4 = 0.0;
      pmat = pmatrices[0][j][k];
      for (m = 0; m <= 19; m++) {
        prodl = 0.0;
        for (l = 0; l <= 19; l++)
          prodl += (pmat[m][l] * x2[l]);
        frexm = x1[m] * freqaa[m];
        prod4 += (prodl * frexm);
      }
      tterm[j] = prod4;
    }
    sumterm = 0.0;
    for (j = 0; j < rcategs; j++)
      sumterm += probcat[j] * tterm[j];
    if (sumterm < 0.0)
        sumterm = 0.00000001;        /* ??? */
    lterm = log(sumterm);
    for (j = 0; j < rcategs; j++)
      clai[j] = tterm[j] / sumterm;
    memcpy(contribution[i], clai, rcategs*sizeof(double));
    if (saveit && !auto_ && usertree)
      l0gf[which - 1][i] = lterm;
    sum += aliasweight[i] * lterm;
  }
  for (j = 0; j < rcategs; j++)
    like[j] = 1.0;
  for (i = 0; i < sites; i++) {
    sumc = 0.0;
    for (k = 0; k < rcategs; k++)
      sumc += probcat[k] * like[k];
    sumc *= lambda;
    if ((ally[i] > 0) && (location[ally[i]-1] > 0)) {
      lai = location[ally[i] - 1];
      memcpy(clai, contribution[lai - 1], rcategs*sizeof(double));
      for (j = 0; j < rcategs; j++)
        nulike[j] = ((1.0 - lambda) * like[j] + sumc) * clai[j];
    } else {
      for (j = 0; j < rcategs; j++)
        nulike[j] = ((1.0 - lambda) * like[j] + sumc);
    }
    memcpy(like, nulike, rcategs*sizeof(double));
  }
  sum2 = 0.0;
  for (i = 0; i < rcategs; i++)
    sum2 += probcat[i] * like[i];
  sum += log(sum2);
  curtree.likelihood = sum;
  if (!saveit || auto_ || !usertree)
    return sum;
  l0gl[which - 1] = sum;
  if (which == 1) {
    maxwhich = 1;
    maxlogl = sum;
    return sum;
  }
  if (sum > maxlogl) {
    maxwhich = which;
    maxlogl = sum;
  }
  return sum;
}  /* prot_evaluate */


void treevaluate()
{
  /* evaluate a user tree */
  long i;

  inittravtree(curtree.start);
  polishing = true;
  smoothit = true;
  for (i = 1; i <= smoothings * 4; i++)
    smooth (curtree.start);
  dummy = prot_evaluate(curtree.start, true);
}  /* treevaluate */


void promlcopy(tree *a, tree *b, long nonodes, long categs)
{
  /* used in proml */
  long i, j=0;
  node *p, *q;

  for (i = 0; i < spp; i++) {
    prot_copynode(a->nodep[i], b->nodep[i], categs);
    if (a->nodep[i]->back) {
      if (a->nodep[i]->back == a->nodep[a->nodep[i]->back->index - 1])
        b->nodep[i]->back = b->nodep[a->nodep[i]->back->index - 1];
      else if (a->nodep[i]->back == a->nodep[a->nodep[i]->back->index - 1]->next
)
        b->nodep[i]->back = b->nodep[a->nodep[i]->back->index - 1]->next;
      else
        b->nodep[i]->back = b->nodep[a->nodep[i]->back->index - 1]->next->next;
    }
    else b->nodep[i]->back = NULL;
  }
  for (i = spp; i < nonodes; i++) {
    p = a->nodep[i];
    q = b->nodep[i];
    for (j = 1; j <= 3; j++) {
      prot_copynode(p, q, categs);
      if (p->back) {
        if (p->back == a->nodep[p->back->index - 1])
          q->back = b->nodep[p->back->index - 1];
        else if (p->back == a->nodep[p->back->index - 1]->next)
          q->back = b->nodep[p->back->index - 1]->next;
        else
          q->back = b->nodep[p->back->index - 1]->next->next;
      }
      else
        q->back = NULL;
      p = p->next;
      q = q->next;
    }
  }
  b->likelihood = a->likelihood;
  b->start = a->start;               /* start used in dnaml only */
  b->root = a->root;                 /* root used in dnamlk only */
}  /* promlcopy */


void proml_re_move(node **p, node **q)
{
  /* remove p and record in q where it was */
  long i;

  /** assumes bifurcations */
  *q = (*p)->next->back;
  hookup(*q, (*p)->next->next->back);
  (*p)->next->back = NULL;
  (*p)->next->next->back = NULL;
  inittrav((*q));
  inittrav((*q)->back);
  i = 1;
  while (i <= smoothings) {
    smooth(*q);
    if (smoothit)
      smooth((*q)->back);
    i++;
  }
}  /* proml_re_move */


void insert_(node *p, node *q, boolean dooinit)
{
  /* Insert q near p */
  long i, j, num_sibs;
  node *r, *sib_ptr;

  r = p->next->next;
  hookup(r, q->back);
  hookup(p->next, q);
  q->v = 0.5 * q->v;
  q->back->v = q->v;
  r->v = q->v;
  r->back->v = r->v;
  p->initialized = false;
  p->next->initialized = false;
  p->next->next->initialized = false;
  if (dooinit) {
    inittrav(p);
    inittrav(q);
    inittrav(q->back);
  }
  i = 1;
  while (i <= smoothings) {
    smooth(p);
    if (!smoothit) {
      if (!p->tip) {
        num_sibs = count_sibs(p);
        sib_ptr  = p;
        for (j=0; j < num_sibs; j++) {
          smooth(sib_ptr->next->back);
          sib_ptr = sib_ptr->next;
        }
      }
    }
    else
      smooth(p->back);
    i++;
  }
}  /* insert_ */


void addtraverse(node *p, node *q, boolean contin)
{
  /* try adding p at q, proceed recursively through tree */
  long i, num_sibs;
  double like, vsave = 0;
  node *qback = NULL, *sib_ptr;

  if (!smoothit) {
    vsave = q->v;
    qback = q->back;
  }
  insert_(p, q, false);
  like = prot_evaluate(p, false);
  if (like > bestyet || bestyet == UNDEFINED) {
    bestyet = like;
    if (smoothit)
      promlcopy(&curtree, &bestree, nonodes2, rcategs);
    else
      qwhere = q;
    succeeded = true;
  }
  if (smoothit)
    promlcopy(&priortree, &curtree, nonodes2, rcategs);
  else {
    hookup (q, qback);
    q->v = vsave;
    q->back->v = vsave;
    curtree.likelihood = bestyet;
  }
  if (!q->tip && contin) {
    num_sibs = count_sibs(q);
    if (q == curtree.start)
      num_sibs++;
    sib_ptr  = q;
    for (i=0; i < num_sibs; i++) {
      addtraverse(p, sib_ptr->next->back, contin);
      sib_ptr = sib_ptr->next;
    }
  }
}  /* addtraverse */


void rearrange(node *p, node *pp)
{
  /* rearranges the tree, globally or locally moving pp around near p */
  long i, num_sibs;
  double v3 = 0, v4 = 0, v5 = 0;
  node *q, *r, *sib_ptr;

  if (!p->tip && !p->back->tip) {
    curtree.likelihood = bestyet;
    if (p->back->next != pp)
      r = p->back->next;
    else
      r = p->back->next->next;
    /* assumes bifurcations? */
    if (!smoothit) {
      v3 = r->v;
      v4 = r->next->v;
      v5 = r->next->next->v;
    }
    else
      promlcopy(&curtree, &bestree, nonodes2, rcategs);
    proml_re_move(&r, &q);
    if (smoothit)
      promlcopy(&curtree, &priortree, nonodes2, rcategs);
    else
      qwhere = q;
    num_sibs = count_sibs (p);
    sib_ptr  = p;
    for (i=0; i < num_sibs; i++) {
      sib_ptr = sib_ptr->next;
      addtraverse(r, sib_ptr->back, (boolean)(global && (nextsp == spp)));
    }
    if (global && nextsp == spp && !succeeded) {
      p = p->back;
      if (!p->tip) {
        num_sibs = count_sibs (p);
        sib_ptr  = p;
        for (i=0; i < num_sibs; i++) {
          sib_ptr = sib_ptr->next;
          addtraverse(r, sib_ptr->back, (boolean)(global && (nextsp == spp)));
        }
      }
      p = p->back;
    }
    if (smoothit)
      promlcopy(&bestree, &curtree, nonodes2, rcategs);
    else {
      insert_(r, qwhere, true);
      if (qwhere == q) {
        r->v = v3;
        r->back->v = v3;
        r->next->v = v4;
        r->next->back->v = v4;
        r->next->next->v = v5;
        r->next->next->back->v = v5;
        curtree.likelihood = bestyet;
      }
      else {
        smoothit = true;
        for (i = 1; i<=smoothings; i++) {
          smooth(r);
          smooth(r->back);
        }
        smoothit = false;
        promlcopy(&curtree, &bestree, nonodes2, rcategs);
      }
    }
    if (global && nextsp == spp && progress) {
      putchar('.');
      fflush(stdout);
    }
  }
  if (!p->tip) {
    num_sibs = count_sibs(p);
    if (p == curtree.start)
      num_sibs++;
    sib_ptr  = p;
    for (i=0; i < num_sibs; i++) {
      sib_ptr = sib_ptr->next;
      rearrange(sib_ptr->back, p);
    }
  }
}  /* rearrange */


void proml_coordinates(node *p, double lengthsum, long *tipy,
                        double *tipmax)
{
  /* establishes coordinates of nodes */
  node *q, *first, *last;
  double xx;

  if (p->tip) {
    p->xcoord = (long)(over * lengthsum + 0.5);
    p->ycoord = (*tipy);
    p->ymin = (*tipy);
    p->ymax = (*tipy);
    (*tipy) += down;
    if (lengthsum > (*tipmax))
      (*tipmax) = lengthsum;
    return;
  }
  q = p->next;
  do {
    xx = q->v;
    if (xx > 100.0)
      xx = 100.0;
    proml_coordinates(q->back, lengthsum + xx, tipy,tipmax);
    q = q->next;
  } while ((p == curtree.start || p != q) &&
           (p != curtree.start || p->next != q));
  first = p->next->back;
  q = p;
  while (q->next != p)
    q = q->next;
  last = q->back;
  p->xcoord = (long)(over * lengthsum + 0.5);
  if (p == curtree.start)
    p->ycoord = p->next->next->back->ycoord;
  else
    p->ycoord = (first->ycoord + last->ycoord) / 2;
  p->ymin = first->ymin;
  p->ymax = last->ymax;
}  /* proml_coordinates */


void proml_printree()
{
  /* prints out diagram of the tree2 */
  long tipy;
  double scale, tipmax;
  long i;

  if (!treeprint)
    return;
  putc('\n', outfile);
  tipy = 1;
  tipmax = 0.0;
  proml_coordinates(curtree.start, 0.0, &tipy, &tipmax);
  scale = 1.0 / (long)(tipmax + 1.000);
  for (i = 1; i <= (tipy - down); i++)
    drawline2(i, scale, curtree);
  putc('\n', outfile);
}  /* proml_printree */


void sigma(node *p, double *sumlr, double *s1, double *s2)
{
  /* compute standard deviation */
  double tt, aa, like, slope, curv;

  prot_slopecurv(p, p->v, &like, &slope, &curv);
  tt = p->v;
  p->v = epsilon;
  p->back->v = epsilon;
  aa = prot_evaluate(p, false);
  p->v = tt;
  p->back->v = tt;
  (*sumlr) = prot_evaluate(p, false) - aa;
  if (curv < -epsilon) {
    (*s1) = p->v + (-slope - sqrt(slope * slope -  3.841 * curv)) / curv;
    (*s2) = p->v + (-slope + sqrt(slope * slope -  3.841 * curv)) / curv;
  }
  else {
    (*s1) = -1.0;
    (*s2) = -1.0;
  }
}  /* sigma */


void describe(node *p)
{
  /* print out information for one branch */
  long i, num_sibs;
  node *q, *sib_ptr;
  double sumlr, sigma1, sigma2;

  if (!p->tip && !p->initialized)
    prot_nuview(p);
  if (!p->back->tip && !p->back->initialized)
    prot_nuview(p->back);
  q = p->back;
  if (q->tip) {
    fprintf(outfile, " ");
    for (i = 0; i < nmlngth; i++)
      putc(nayme[q->index-1][i], outfile);
    fprintf(outfile, "    ");
  } else
    fprintf(outfile, "  %4ld          ", q->index - spp);
  if (p->tip) {
    for (i = 0; i < nmlngth; i++)
      putc(nayme[p->index-1][i], outfile);
  } else
    fprintf(outfile, "%4ld      ", p->index - spp);
  fprintf(outfile, "%15.5f", q->v);
  if (p->iter) {
    sigma(q, &sumlr, &sigma1, &sigma2);
    if (sigma1 <= sigma2)
      fprintf(outfile, "     (     zero,    infinity)");
    else {
      fprintf(outfile, "     (");
      if (sigma2 <= 0.0)
        fprintf(outfile, "     zero");
      else
        fprintf(outfile, "%9.5f", sigma2);
      fprintf(outfile, ",%12.5f", sigma1);
      putc(')', outfile);
      }
    if (sumlr > 1.9205)
      fprintf(outfile, " *");
    if (sumlr > 2.995)
      putc('*', outfile);
    }
  putc('\n', outfile);
  if (!p->tip) {
    num_sibs = count_sibs(p);
    sib_ptr  = p;
    for (i=0; i < num_sibs; i++) {
      sib_ptr = sib_ptr->next;
      describe(sib_ptr->back);
    }
  }
}  /* describe */


void prot_reconstr(node *p, long n)
{
  /* reconstruct and print out acid at site n+1 at node p */
  long i, j, k, first, num_sibs = 0;
  double f, sum, xx[20];
  node *q = NULL;

  if (p->tip)
    putc(y[p->index-1][n], outfile);
  else {
    num_sibs = count_sibs(p);
    if ((ally[n] == 0) || (location[ally[n]-1] == 0))
      putc('.', outfile);
    else {
      j = location[ally[n]-1] - 1;
      sum = 0;
      for (i = 0; i <= 19; i++) {
        f = p->protx[j][mx-1][i];
        if (!p->tip) {
          q = p;
          for (k = 0; k < num_sibs; k++) {
            q = q->next;
            f *= q->protx[j][mx-1][i];
          }
        }
        f = sqrt(f);
        xx[i] = f * freqaa[i];
        sum += xx[i];
      }
      for (i = 0; i <= 19; i++)
        xx[i] /= sum;
      first = 0;
      for (i = 0; i <= 19; i++)
        if (xx[i] > xx[first])
          first = i;
      if (xx[first] > 0.95)
        putc(aachar[first], outfile);
      else
        putc(tolower(aachar[first]), outfile);
      if (rctgry && rcategs > 1)
        mx = mp[n][mx - 1];
      else
        mx = 1;
    }
  }
} /* prot_reconstr */


void rectrav(node *p, long m, long n)
{
  /* print out segment of reconstructed sequence for one branch */
  long i;

  putc(' ', outfile);
  if (p->tip) {
    for (i = 0; i < nmlngth; i++)
      putc(nayme[p->index-1][i], outfile);
  } else
    fprintf(outfile, "%4ld      ", p->index - spp);
  fprintf(outfile, "  ");
  mx = mx0;
  for (i = m; i <= n; i++) {
    if ((i % 10 == 0) && (i != m))
      putc(' ', outfile);
    prot_reconstr(p, i);
  }
  putc('\n', outfile);
  if (!p->tip) {
    rectrav(p->next->back, m, n);
    rectrav(p->next->next->back, m, n);
  }
  mx1 = mx;
}  /* rectrav */


void summarize()
{
  /* print out branch length information and node numbers */
  long i, j, mm, num_sibs;
  double mode, sum;
  double like[maxcategs],nulike[maxcategs];
  double **marginal;
  node   *sib_ptr;

  if (!treeprint)
    return;
  fprintf(outfile, "\nremember: ");
  if (outgropt)
    fprintf(outfile, "(although rooted by outgroup) ");
  fprintf(outfile, "this is an unrooted tree!\n\n");
  fprintf(outfile, "Ln Likelihood = %11.5f\n", curtree.likelihood);
  fprintf(outfile, "\n Between        And            Length");
  if (!(usertree && lngths))
    fprintf(outfile, "      Approx. Confidence Limits");
  fprintf(outfile, "\n");
  fprintf(outfile, " -------        ---            ------");
  if (!(usertree && lngths))
    fprintf(outfile, "      ------- ---------- ------");
  fprintf(outfile, "\n\n");
  for (i = spp; i < nonodes2; i++) {
    /* So this works with arbitrary multifurcations */
    if (curtree.nodep[i]) {
      num_sibs = count_sibs (curtree.nodep[i]);
      sib_ptr  = curtree.nodep[i];
      for (j = 0; j < num_sibs; j++) {
        sib_ptr->initialized = false;
        sib_ptr              = sib_ptr->next;
      }
    }
  }

  describe(curtree.start->back);

  /* So this works with arbitrary multifurcations */
  num_sibs = count_sibs(curtree.start);
  sib_ptr  = curtree.start;
  for (i=0; i < num_sibs; i++) {
    sib_ptr = sib_ptr->next;
    describe(sib_ptr->back);
  }

  fprintf(outfile, "\n");
  if (!(usertree && lngths)) {
    fprintf(outfile, "     *  = significantly positive, P < 0.05\n");
    fprintf(outfile, "     ** = significantly positive, P < 0.01\n\n");
  }
  dummy = prot_evaluate(curtree.start, false);
  if (rctgry && rcategs > 1) {
    for (i = 0; i < rcategs; i++)
      like[i] = 1.0;
    for (i = sites - 1; i >= 0; i--) {
      sum = 0.0;
      for (j = 0; j < rcategs; j++) {
        nulike[j] = (1.0 - lambda + lambda * probcat[j]) * like[j];
        mp[i][j] = j + 1;
        for (k = 1; k <= rcategs; k++) {
          if (k != j + 1) {
            if (lambda * probcat[k - 1] * like[k - 1] > nulike[j]) {
              nulike[j] = lambda * probcat[k - 1] * like[k - 1];
              mp[i][j] = k;
            }
          }
        }
        if ((ally[i] > 0) && (location[ally[i]-1] > 0))
          nulike[j] *= contribution[location[ally[i] - 1] - 1][j];
        sum += nulike[j];
      }
      for (j = 0; j < rcategs; j++)
        nulike[j] /= sum;
      memcpy(like, nulike, rcategs * sizeof(double));
    }
    mode = 0.0;
    mx = 1;
    for (i = 1; i <= rcategs; i++) {
      if (probcat[i - 1] * like[i - 1] > mode) {
        mx = i;
        mode = probcat[i - 1] * like[i - 1];
      }
    }
    mx0 = mx;
    fprintf(outfile,
 "Combination of categories that contributes the most to the likelihood:\n\n");
    for (i = 1; i <= nmlngth + 3; i++)
      putc(' ', outfile);
    for (i = 1; i <= sites; i++) {
      fprintf(outfile, "%ld", mx);
      if (i % 10 == 0)
        putc(' ', outfile);
      if (i % 60 == 0 && i != sites) {
        putc('\n', outfile);
        for (j = 1; j <= nmlngth + 3; j++)
          putc(' ', outfile);
      }
      mx = mp[i - 1][mx - 1];
    }
    fprintf(outfile, "\n\n");
    marginal = (double **) Malloc(sites*sizeof(double *));
    for (i = 0; i < sites; i++)
      marginal[i] = (double *) Malloc(rcategs*sizeof(double));
    for (i = 0; i < rcategs; i++)
      like[i] = 1.0;
    for (i = sites - 1; i >= 0; i--) {
      sum = 0.0;
      for (j = 0; j < rcategs; j++) {
        nulike[j] = (1.0 - lambda + lambda * probcat[j]) * like[j];
        for (k = 1; k <= rcategs; k++) {
          if (k != j + 1)
              nulike[j] += lambda * probcat[k - 1] * like[k - 1];
        }
        if ((ally[i] > 0) && (location[ally[i]-1] > 0))
          nulike[j] *= contribution[location[ally[i] - 1] - 1][j];
        sum += nulike[j];
      }
      for (j = 0; j < rcategs; j++) {
        nulike[j] /= sum;
        marginal[i][j] = nulike[j];
      }
      memcpy(like, nulike, rcategs * sizeof(double));
    }
    for (i = 0; i < rcategs; i++)
      like[i] = 1.0;
    for (i = 0; i < sites; i++) {
      sum = 0.0;
      for (j = 0; j < rcategs; j++) {
        nulike[j] = (1.0 - lambda + lambda * probcat[j]) * like[j];
        for (k = 1; k <= rcategs; k++) {
          if (k != j + 1)
              nulike[j] += lambda * probcat[k - 1] * like[k - 1];
        }
        marginal[i][j] *= like[j] * probcat[j];
        sum += nulike[j];
      }
      for (j = 0; j < rcategs; j++)
        nulike[j] /= sum;
      memcpy(like, nulike, rcategs * sizeof(double));
      sum = 0.0;
      for (j = 0; j < rcategs; j++)
        sum += marginal[i][j];
      for (j = 0; j < rcategs; j++)
        marginal[i][j] /= sum;
    }
    fprintf(outfile, "Most probable category at each site if > 0.95");
    fprintf(outfile, " probability (\".\" otherwise)\n\n");
    for (i = 1; i <= nmlngth + 3; i++)
      putc(' ', outfile);
    for (i = 0; i < sites; i++) {
      sum = 0.0;
      for (j = 0; j < rcategs; j++)
        if (marginal[i][j] > sum) {
          sum = marginal[i][j];
          mm = j;
        }
        if (sum >= 0.95)
        fprintf(outfile, "%ld", mm+1);
      else
        putc('.', outfile);
      if ((i+1) % 60 == 0) {
        if (i != 0) {
          putc('\n', outfile);
          for (j = 1; j <= nmlngth + 3; j++)
            putc(' ', outfile);
        }
      }
      else if ((i+1) % 10 == 0)
        putc(' ', outfile);
    }
    putc('\n', outfile);
    for (i = 0; i < sites; i++)
      free(marginal[i]);
    free(marginal);
  }
  putc('\n', outfile);
  if (hypstate) {
    fprintf(outfile, "Probable sequences at interior nodes:\n\n");
    fprintf(outfile, "  node       ");
    for (i = 0; (i < 13) && (i < ((sites + (sites-1)/10 - 39) / 2)); i++)
      putc(' ', outfile);
    fprintf(outfile, "Reconstructed sequence (caps if > 0.95)\n\n");
    if (!rctgry || (rcategs == 1))
      mx0 = 1;
    for (i = 0; i < sites; i += 60) {
      k = i + 59;
      if (k >= sites)
        k = sites - 1;
      rectrav(curtree.start, i, k);
      rectrav(curtree.start->back, i, k);
      putc('\n', outfile);
      mx0 = mx1;
    }
  }
}  /* summarize */


void initpromlnode(node **p, node **grbg, node *q, long len, long nodei,
                        long *ntips, long *parens, initops whichinit,
                        pointarray treenode, pointarray nodep, Char *str,
                        Char *ch, FILE *intree)
{
  /* initializes a node */
  boolean minusread;
  double valyew, divisor;

  switch (whichinit) {
  case bottom:
    gnu(grbg, p);
    (*p)->index = nodei;
    (*p)->tip = false;
    malloc_ppheno((*p), endsite, rcategs);
    nodep[(*p)->index - 1] = (*p);
    break;
  case nonbottom:
    gnu(grbg, p);
    malloc_ppheno(*p, endsite, rcategs);
    (*p)->index = nodei;
    break;
  case tip:
    match_names_to_data(str, nodep, p, spp);
    break;
  case iter:
    (*p)->initialized = false;
    (*p)->v = initialv;
    (*p)->iter = true;
    if ((*p)->back != NULL)
      (*p)->back->iter = true;
    break;
  case length:
    processlength(&valyew, &divisor, ch, &minusread, intree, parens);
    (*p)->v = valyew / divisor;
    (*p)->iter = false;
    if ((*p)->back != NULL) {
      (*p)->back->v = (*p)->v;
      (*p)->back->iter = false;
    }
    break;
  default:      /* cases hslength, hsnolength, treewt, unittrwt */
    break;      /* should never occur                           */
  }
} /* initpromlnode */


void dnaml_treeout(node *p)
{
  /* write out file with representation of final tree2 */
  /* Only works for bifurcations! */
  long i, n, w;
  Char c;
  double x;

  if (p->tip) {
    n = 0;
    for (i = 1; i <= nmlngth; i++) {
      if (nayme[p->index-1][i - 1] != ' ')
        n = i;
    }
    for (i = 0; i < n; i++) {
      c = nayme[p->index-1][i];
      if (c == ' ')
        c = '_';
      putc(c, outtree);
    }
    col += n;
  } else {
    putc('(', outtree);
    col++;
    dnaml_treeout(p->next->back);
    putc(',', outtree);
    col++;
    if (col > 45) {
      putc('\n', outtree);
      col = 0;
    }
    dnaml_treeout(p->next->next->back);
    if (p == curtree.start) {
      putc(',', outtree);
      col++;
      if (col > 45) {
        putc('\n', outtree);
        col = 0;
      }
      dnaml_treeout(p->back);
    }
    putc(')', outtree);
    col++;
  }
  x = p->v;
  if (x > 0.0)
    w = (long)(0.43429448222 * log(x));
  else if (x == 0.0)
    w = 0;
  else
    w = (long)(0.43429448222 * log(-x)) + 1;
  if (w < 0)
    w = 0;
  if (p == curtree.start)
    fprintf(outtree, ";\n");
  else {
    fprintf(outtree, ":%*.5f", (int)(w + 7), x);
    col += w + 8;
  }
}  /* dnaml_treeout */


void buildnewtip(long m, tree *tr)
{
  node *p;

  p = tr->nodep[nextsp + spp - 3];
  hookup(tr->nodep[m - 1], p);
  p->v = initialv;
  p->back->v =  initialv;
}  /* buildnewtip */


void buildsimpletree(tree *tr)
{
  hookup(tr->nodep[enterorder[0] - 1], tr->nodep[enterorder[1] - 1]);
  tr->nodep[enterorder[0] - 1]->v = 1.0;
  tr->nodep[enterorder[0] - 1]->back->v = 1.0;
  tr->nodep[enterorder[1] - 1]->v = 1.0;
  tr->nodep[enterorder[1] - 1]->back->v = 1.0;
  buildnewtip(enterorder[2], tr);
  insert_(tr->nodep[enterorder[2] - 1]->back,
          tr->nodep[enterorder[0] - 1], false);
}  /* buildsimpletree */


void free_all_protx (long nonodes, pointarray treenode)
{
  /* used in proml */
  long i, j, k;
  node *p;

  /* Zero thru spp are tips, */
  for (i = 0; i < spp; i++) {
    for (j = 0; j < endsite; j++)
      free(treenode[i]->protx[j]);
    free(treenode[i]->protx);
  }

  /* The rest are rings (i.e. triads) */
  for (i = spp; i < nonodes; i++) {
    if (treenode[i] != NULL) {
      p = treenode[i];
      for (j = 1; j <= 3; j++) {
        for (k = 0; k < endsite; k++)
          free(p->protx[k]);
        free(p->protx);
        p = p->next;
      }
    }
  }
}  /* free_all_protx */


void maketree()
{
  long i, j, k, l;
  long num_sibs;
  boolean dummy_haslengths, dummy_first, goteof;
  pointarray dummy_treenode=NULL;
  long nextnode;
  node *root, *q, *r;

  prot_inittable();

  if (usertree) {
    openfile(&intree,INTREE,"input tree file", "r",progname,intreename);
    numtrees = countsemic(&intree);
    if (numtrees > 2)
      initseed(&inseed, &inseed0, seed);
    l0gl = (double *) Malloc(numtrees * sizeof(double));
    l0gf = (double **) Malloc(numtrees * sizeof(double *));
    for (i=0; i < numtrees; ++i)
      l0gf[i] = (double *) Malloc(endsite * sizeof(double));
    if (treeprint) {
      fprintf(outfile, "User-defined tree");
      if (numtrees > 1)
        putc('s', outfile);
      fprintf(outfile, ":\n\n");
    }
    which = 1;

    /* This taken out of tree read, used to be [spp-1], but referring
       to [0] produces output identical to what the pre-modified dnaml
       produced. */

    while (which <= numtrees) {

      /* These initializations required each time through the loop
         since multiple trees require re-initialization */
      dummy_haslengths = true;
      nextnode         = 0;
      dummy_first      = true;
      goteof           = false;

      treeread(intree, &root, dummy_treenode, &goteof, &dummy_first,
               curtree.nodep, &nextnode,
               &dummy_haslengths, &grbg, initpromlnode);
      q = root;
      r = root;
      while (!(q->next == root))
        q = q->next;
      q->next = root->next;
      root = q;
      chuck(&grbg, r);
      curtree.nodep[spp] = q;
      if (goteof && (which <= numtrees)) {
        /* if we hit the end of the file prematurely */
        printf ("\n");
        printf ("ERROR: trees missing at end of file.\n");
        printf ("\tExpected number of trees:\t\t%ld\n", numtrees);
        printf ("\tNumber of trees actually in file:\t%ld.\n\n", which - 1);
        exxit (-1);
      }

      curtree.start = curtree.nodep[0]->back;

      treevaluate();
      if (reconsider) {
        bestyet = UNDEFINED;
        succeeded = true;
        while (succeeded) {
          succeeded = false;
          rearrange(curtree.start, curtree.start->back);
        }
        treevaluate();
      }
      proml_printree();
      summarize();
      if (trout) {
        col = 0;
        dnaml_treeout(curtree.start);
      }
      which++;
    }
    FClose(intree);
    putc('\n', outfile);
    if (!auto_ && numtrees > 1 && weightsum > 1 )
      standev2(numtrees, maxwhich, 0, endsite-1, maxlogl,
               l0gl, l0gf, aliasweight, seed);
  } else {
    /* If there's no input user tree, */
    for (i = 1; i <= spp; i++)
      enterorder[i - 1] = i;
    if (jumble)
      randumize(seed, enterorder);
    if (progress) {
      printf("\nAdding species:\n");
      writename(0, 3, enterorder);
#ifdef WIN32
      phyFillScreenColor();
#endif
    }
    nextsp = 3;
    polishing = false;
    buildsimpletree(&curtree);
    curtree.start = curtree.nodep[enterorder[0] - 1]->back;
    smoothit = improve;
    nextsp = 4;
    while (nextsp <= spp) {
      buildnewtip(enterorder[nextsp - 1], &curtree);
      bestyet = UNDEFINED;
      if (smoothit)
        promlcopy(&curtree, &priortree, nonodes2, rcategs);
      addtraverse(curtree.nodep[enterorder[nextsp - 1] - 1]->back,
                  curtree.start, true);
      if (smoothit)
        promlcopy(&bestree, &curtree, nonodes2, rcategs);
      else {
        insert_(curtree.nodep[enterorder[nextsp - 1] - 1]->back, qwhere, true);
        smoothit = true;
        for (i = 1; i<=smoothings; i++) {
          smooth(curtree.start);
          smooth(curtree.start->back);
        }
        smoothit = false;
        promlcopy(&curtree, &bestree, nonodes2, rcategs);
        bestyet = curtree.likelihood;
      }
      if (progress) {
        writename(nextsp - 1, 1, enterorder);
#ifdef WIN32
        phyFillScreenColor();
#endif
      }
      if (global && nextsp == spp && progress) {
        printf("Doing global rearrangements\n");
        printf("  !");
        for (j = 1; j <= (spp - 3); j++)
          putchar('-');
        printf("!\n");
#ifdef WIN32
        phyFillScreenColor();
#endif
      }
      succeeded = true;
      while (succeeded) {
        succeeded = false;
        if (global && nextsp == spp && progress) {
          printf("   ");
          fflush(stdout);
        }
        rearrange(curtree.start, curtree.start->back);
        if (global && nextsp == spp && progress)
          putchar('\n');
      }
      for (i = spp; i < nextsp + spp - 2; i++) {
        curtree.nodep[i]->initialized = false;
        curtree.nodep[i]->next->initialized = false;
        curtree.nodep[i]->next->next->initialized = false;
      }
      if (!smoothit) {
        smoothit = true;
        for (i = 1; i<=smoothings; i++) {
          smooth(curtree.start);
          smooth(curtree.start->back);
        }
        smoothit = false;
        promlcopy(&curtree, &bestree, nonodes2, rcategs);
        bestyet = curtree.likelihood;
      }
      nextsp++;
    }
    if (global && progress) {
      putchar('\n');
      fflush(stdout);
#ifdef WIN32
      phyFillScreenColor();
#endif
    }
    if (njumble > 1) {
      if (jumb == 1)
        promlcopy(&bestree, &bestree2, nonodes2, rcategs);
      else
        if (bestree2.likelihood < bestree.likelihood)
          promlcopy(&bestree, &bestree2, nonodes2, rcategs);
    }
    if (jumb == njumble) {
      if (njumble > 1)
        promlcopy(&bestree2, &curtree, nonodes2, rcategs);
      curtree.start = curtree.nodep[outgrno - 1]->back;
      for (i = 0; i < nonodes2; i++) {
        if (i < spp)
          curtree.nodep[i]->initialized = false;
        else {
          curtree.nodep[i]->initialized = false;
          curtree.nodep[i]->next->initialized = false;
          curtree.nodep[i]->next->next->initialized = false;
        }
      }
      treevaluate();
      proml_printree();
      summarize();
      if (trout) {
        col = 0;
        dnaml_treeout(curtree.start);
      }
    }
  }
  if (usertree) {
    free(l0gl);
    for (i=0; i < numtrees; i++)
      free(l0gf[i]);
    free(l0gf);
  }
  for (num_sibs = 0; num_sibs < max_num_sibs; num_sibs++) {
    for (j = 0; j < rcategs; j++) {
      for (k = 0; k < categs; k++) {
        for (l = 0; l < 20; l++) {
          free(pmatrices[num_sibs][j][k][l]);
        }
        free(pmatrices[num_sibs][j][k]);
      }
     free(pmatrices[num_sibs][j]);
   }
   free(pmatrices[num_sibs]);
  }
  if (jumb < njumble)
    return;
  free(contribution);
  free(mp);
  for (i=0; i < endsite; i++)
     free(term[i]);
  free(term);
  for (i=0; i < endsite; i++)
     free(slopeterm[i]);
  free(slopeterm);
  for (i=0; i < endsite; i++)
     free(curveterm[i]);
  free(curveterm);
  free_all_protx(nonodes2, curtree.nodep);
  if (!usertree || reconsider) {
    free_all_protx(nonodes2, bestree.nodep);
    free_all_protx(nonodes2, priortree.nodep);
    if (njumble > 1)
      free_all_protx(nonodes2, bestree2.nodep);
  }
  if (progress) {
    printf("\n\nOutput written to file \"%s\"\n\n", outfilename);
    if (trout)
      printf("Tree also written onto file \"%s\"\n", outtreename);
    putchar('\n');
  }
}  /* maketree */


void clean_up()
{
  /* Free and/or close stuff */
  long i;

    free (rrate);
    free (probcat);
    free (rate);
    /* Seems to require freeing every time... */
    for (i = 0; i < spp; i++) {
      free (y[i]);
    }
    free (y);
    free (nayme);
    free (enterorder);
    free (category);
    free (weight);
    free (alias);
    free (ally);
    free (location);
    free (aliasweight);
    free (probmat);
    free (eigmat);

#if 0          /* ???? debug ???? */
  freetree2(curtree.nodep, nonodes2);

  if (! (usertree && !reconsider)) {
    freetree2(bestree.nodep, nonodes2);
    freetree2(priortree.nodep, nonodes2);
  }

  if (! (njumble <= 1))
    freetree2(bestree2.nodep, nonodes2);
#endif
  FClose(infile);
  FClose(outfile);
  FClose(outtree);
#ifdef MAC
  fixmacfile(outfilename);
  fixmacfile(outtreename);
#endif
}   /* clean_up */


int main(int argc, Char *argv[])
{  /* Protein Maximum Likelihood */
#ifdef MAC
  argc = 1;             /* macsetup("ProML","");        */
  argv[0] = "ProML";
#endif
  init(argc,argv);
  progname = argv[0];
  openfile(&infile,INFILE,"input file","r",argv[0],infilename);
  openfile(&outfile,OUTFILE,"output file","w",argv[0],outfilename);
  mulsets = false;
  datasets = 1;
  firstset = true;
  ibmpc = IBMCRT;
  ansi = ANSICRT;
  grbg = NULL;
  doinit();
  if (ctgry)
    openfile(&catfile,CATFILE,"categories file","r",argv[0],catfilename);
  if (weights || justwts)
    openfile(&weightfile,WEIGHTFILE,"weights file","r",argv[0],weightfilename);
  if (trout)
    openfile(&outtree,OUTTREE,"output tree file","w",argv[0],outtreename);
  for (ith = 1; ith <= datasets; ith++) {
    if (datasets > 1) {
      fprintf(outfile, "Data set # %ld:\n", ith);
      printf("\nData set # %ld:\n", ith);
    }
    getinput();
    if (ith == 1)
      firstset = false;
    for (jumb = 1; jumb <= njumble; jumb++) {
      max_num_sibs = 0;
      maketree();
    }
  }

  clean_up();
  printf("Done.\n\n");
#ifdef WIN32
  phyRestoreConsoleAttributes();
#endif
  return 0;
}  /* Protein Maximum Likelihood */

